"""
Filename:     plot.py
Author:       Shannon Mason, shannon.mason@ecmwf.int
Description:  Plotting functions
"""

import pandas as pd
import numpy as np

#For loading and handling netCDF data
import xarray as xr

#Use seaborn to control matplotlib visual style
import matplotlib.pyplot as plt
import seaborn as sns

#For log-scaled colormaps
from matplotlib.colors import LogNorm#, DivergingNorm

#Plot formatting functions
from ecradplot.general import *

#I/O functions
from ecradplot.io import *
import os

#### Set plotting style ####
sns.set_style('ticks')
sns.set_context('poster')

def warn(*args, **kwargs):
    pass
import warnings
warnings.warn = warn
warnings.simplefilter(action = "ignore", category = RuntimeWarning)


def get_vextents(da, q=0.01, symmetric=False):
    vmin = da.quantile(q=0.01).values
    vmax = da.quantile(q=1-q).values

    #All negative
    if (vmin < 0) & (vmax < 0) & ~symmetric:
        return vmin, 0
    #All positive
    elif (vmin > 0) & (vmax > 0) & ~symmetric:
        return 0, vmax
    else:
        v = max(np.abs(vmin), np.abs(vmax))
        return -1*v, v

def irregular_pcolor(ax, X, Y, C, args, cbar_kwargs=None):
    _X, _Y, _C = xr.broadcast(X, Y, C)
    _cm = ax.pcolor(_X.values, _Y.values, _C.values, **args)

    if cbar_kwargs:
        try:
            _cb = plt.colorbar(_cm, ax=ax, **cbar_kwargs)
        except:
            print("Bug with colorbars")

    if 'level' in C.dims:
        ax.fill_between(_X.max('level'), _Y.max('level'), y2=1100e2, facecolor='0.67',
                        hatch='////', edgecolor='k', lw=0.0, zorder=-10)
    elif 'half_level' in C.dims:
        ax.fill_between(_X.max('half_level'), _Y.max('half_level'), y2=1100e2, facecolor='0.67',
                        hatch='////', edgecolor='k', lw=0.0, zorder=-10)
    return _cm

def irregular_contour(ax, X, Y, C, args, cbar_kwargs=None):
    _X, _Y, _C = xr.broadcast(X, Y, C)
    _cm = ax.contour(_X, _Y, _C, **args)
    if cbar_kwargs:
        _cb = plt.colorbar(_cm, ax=ax, **cbar_kwargs)
    return _cm

def add_temperature_contours(ax, ds, x_dim='latitude'):
    """
    Draw contours of temperature (from ds) to ax.
    """

    _cn = irregular_contour(ax, ds.latitude, ds.pressure_hl, ds.temperature_hl-273.15,
                     dict(levels=np.arange(-80,41,20),
                          colors=['k'],
                          linewidths=[1.5, 0.5, 1.5, 0.5, 2.5, 0.5, 1.5]))
    _labels = ax.clabel(_cn, [l for l in [-80,-60,-40,-20,0,20,40] if l in _cn.levels], inline=1, fmt='$%.0f^{\circ}$C', fontsize='xx-small', colors=['k'])

    for l in _labels:
        l.set_rotation(0)


def plot_inputs_noncloud(IFS_srcfile, dstfile=None, line_ds='default'):
    """
    Plot multiple-panel figure describing non-cloud inputs to ecRAD.
    """

    _ds = load_inputs(IFS_srcfile)

    #Set up figure and axes
    nrows=4

    fig, axes= plt.subplots(figsize=(25,4*nrows), nrows=nrows, sharex=True)

    #First panel: SW & surface fields
    i=0
    _ds.cos_solar_zenith_angle.where(_ds.cos_solar_zenith_angle >= 0).plot(ax=axes[i], x='latitude', color='k', lw=3)
    axes[i].set_xlabel('')
    axes[i].set_ylabel(r'$\cos \theta_s$ [-]')
    axes[i].set_yticks([0,0.5,1.])
    axes[i].set_title('Solar zenith angle and shortwave albedo')
    if 'solar_irradiance' in _ds:
        axes[i].text(0.001, 1.01, f"Solar irradiance\n$Q={_ds.solar_irradiance.values:5.1f}$ W m$^{{-2}}$", ha='left', va='bottom', fontsize='small', transform=axes[i].transAxes)
    _ax0 = axes[i].twinx()
    _ax0.yaxis.set_label_position("right")
    _ax0.yaxis.tick_right()

    if hasattr(_ds, 'sw_albedo_band'):
        _ds.sw_albedo.isel(sw_albedo_band=2).plot.step(ax=_ax0, x='latitude', color=sns.color_palette()[0], lw=4, drawstyle=line_ds)
    else:
        _ds.sw_albedo.plot(ax=_ax0, x='latitude', color=sns.color_palette()[0], lw=4, drawstyle=line_ds)
    _ax0.set_yticks([0,0.5,1.0])
    _ax0.set_yticklabels([0,0.5,1.0],  color=sns.color_palette()[0])
    _ax0.set_ylabel(r'$\alpha_{SW}$ [-]', color=sns.color_palette()[0])

    #Second panel: LW surface fields
    i+=1
    _ds.skin_temperature.plot(ax=axes[i], x='latitude', color='k', lw=3)
    axes[i].set_xlabel('')
    axes[i].set_ylabel(r'$T_s$ [K]')
    axes[i].set_title('Skin temperature and longwave emissivity')

    _ax1 = axes[i].twinx()
    _ax1.yaxis.set_label_position("right")
    _ax1.yaxis.tick_right()
    if hasattr(_ds, 'lw_emissivity_band'):
        _ds.lw_emissivity.isel(lw_emissivity_band=1).plot.step(ax=_ax1, x='latitude', color=sns.color_palette()[3], lw=4, drawstyle=line_ds)
    else:
        _ds.lw_emissivity.plot(ax=_ax1, x='latitude', color=sns.color_palette()[3], lw=4, drawstyle=line_ds)
    _ax1.set_yticks([0.9,0.95,1.0])
    _ax1.set_ylim(0.89,1.0)
    _ax1.set_yticklabels([0.9,0.95,1.0],  color=sns.color_palette()[3])
    _ax1.set_ylabel(r'$\epsilon_{LW}$ [-]', color=sns.color_palette()[3])

    #Specific humidity
    i+=1
    irregular_pcolor(axes[i], _ds.latitude, _ds.pressure_fl, _ds.q,
                     dict(norm=LogNorm(1e-6, 1e-2), cmap='Greens'),
                     cbar_kwargs={'pad':0.01, 'label':'mass mixing ratio\n[kg kg$^{-1}$]', 'ticks':[1e-5,1e-4,1e-3,1e-2]})

    axes[i].set_title('Specific humidity')
    axes[i].set_xlabel('')

    axes[i].set_yscale('linear')
    axes[i].set_yticks([1000e2,800e2,600e2,400e2,200e2,1])
    axes[i].set_ylim(1050e2,1)

    #Ozone
    i+=1
    irregular_pcolor(axes[i], _ds.latitude, _ds.pressure_fl, _ds.o3_mmr,
                     dict(norm=LogNorm(1e-8, 1e-5), cmap='Blues'),
                     cbar_kwargs={'pad':0.01, 'label':'mass mixing ratio\n[kg kg$^{-1}$]', 'ticks':[1e-8,1e-7,1e-6,1e-5]})

    axes[i].set_title('Ozone')
    axes[i].set_xlabel('')

    axes[i].set_yscale('log')
    axes[i].set_yticks([1e5,1e4,1e3,1e2,1e1,1e0])
    axes[i].set_ylim(1.1e5,1)

    for ax in axes[-2:]:
        add_temperature_contours(ax, _ds)
        format_pressure(ax)

    for ax in axes[:-2]:
        snap_to_axis(ax, axes[-1])

    axes[-1].set_xlim(-90,90)
    axes[-1].set_xticks(np.arange(-90,91,15))
    format_latitude(axes[-1])
    axes[-1].set_xlabel('Latitude')

    add_subfigure_labels(axes)

    if hasattr(_ds, 'experiment'):
        fig.suptitle(_ds.attrs['experiment'] + "\nsurface properties and atmospheric composition", x=get_figure_center(axes[0]), y=get_figure_top(fig, axes[0]), va='bottom')
    else:
        fig.suptitle("surface properties and atmospheric composition", x=get_figure_center(axes[0]), y=get_figure_top(fig, axes[0]), va='bottom')

    if dstfile:
        fig.savefig(dstfile, dpi=90, bbox_inches='tight')
    else:
        return fig, axes


def plot_inputs_cloud(IFS_srcfile, include_effective_radius=False, dstfile=None):
    _ds = load_inputs(IFS_srcfile)

    if include_effective_radius:
        nrows=5
    else:
        nrows=3

    fig, axes = plt.subplots(figsize=(25,4*nrows), nrows=nrows, sharex=True, sharey=True,  subplot_kw={'facecolor':sns.xkcd_rgb['earth']})

    irregular_pcolor(axes[0], _ds.latitude, _ds.pressure_fl, _ds.cloud_fraction,
                     dict(vmin=0, vmax=1, cmap='gray_r'),
                     cbar_kwargs={'pad':0.01, 'label':'fraction', 'ticks':[0, 0.2, 0.4, 0.6, 0.8, 1.0]})
    axes[0].set_title('Cloud fraction')
    axes[0].set_xlabel('')

    irregular_pcolor(axes[1], _ds.latitude, _ds.pressure_fl, _ds.q_ice.where(_ds.q_ice > 1e-10).fillna(1e-10),
                     dict(norm=LogNorm(1e-8, 0.5e-2), cmap='Blues'),
                     cbar_kwargs={'pad':0.01, 'label':'mixing ratio\n[kg kg$^{-1}$]', 'ticks':[1e-7, 1e-5, 1e-3]})
    axes[1].set_title('Cloud ice water content')
    axes[1].set_xlabel('')

    irregular_pcolor(axes[2], _ds.latitude, _ds.pressure_fl, _ds.q_liquid.where(_ds.q_liquid > 1e-10).fillna(1e-10),
                     dict(norm=LogNorm(1e-8, 0.5e-2), cmap='Reds'),
                     cbar_kwargs={'pad':0.01, 'label':'mixing ratio\n[kg kg$^{-1}$]', 'ticks':[1e-7, 1e-5, 1e-3]})
    axes[2].set_title('Cloud liquid water content')
    format_latitude(axes[-1])

    if include_effective_radius:
        axes[2].set_xlabel('')

        irregular_pcolor(axes[3], _ds.latitude, _ds.pressure_fl, _ds.re_ice.where(_ds.q_ice > 1e-10).fillna(1e-10),
                         dict(norm=LogNorm(3e-6, 1e-4), cmap='Blues'),
                         cbar_kwargs={'pad':0.01, 'label':'$r_{\mathrm{eff}}$ [m]'})
        axes[3].set_title('Ice effective radius')
        axes[3].set_xlabel('')

        irregular_pcolor(axes[4], _ds.latitude, _ds.re_liquid.where(_ds.q_liquid > 1e-10).fillna(1e-10),
                         dict(norm=LogNorm(3e-6, 1e-4), cmap='Reds'),
                         cbar_kwargs={'pad':0.01, 'label':'$r_{\mathrm{eff}}$ [m]'})
        axes[4].set_title('Liquid effective radius')

    for ax in axes:
        add_temperature_contours(ax, _ds)
        format_pressure(ax)

    axes[-1].set_xlim(-90,90)
    axes[-1].set_xticks(np.arange(-90,91,15))
    axes[-1].set_xlabel('Latitude')

    axes[0].set_yticks([1000e2,800e2,600e2,400e2,200e2,1])
    axes[0].set_ylim(1050e2,1)

    add_subfigure_labels(axes)

    if hasattr(_ds, 'experiment'):
        fig.suptitle(_ds.attrs['experiment'] + "\ncloud fields", x=get_figure_center(axes[0]), y=get_figure_top(fig, axes[0])+ 0.1, va='bottom')
    else:
        fig.suptitle("cloud fields", x=get_figure_center(axes[0]), y=get_figure_top(fig, axes[0])+ 0.1, va='bottom')

    if dstfile:
        fig.savefig(dstfile, dpi=90, bbox_inches='tight')
    else:
        return fig, axes

def plot_inputs_aerosols(IFS_srcfile, dstfile=None):
    _ds = load_inputs(IFS_srcfile)

    nrows=5

    fig, axes = plt.subplots(figsize=(25,4*nrows), nrows=nrows, sharex=True, sharey=True, )

    irregular_pcolor(axes[0], _ds.latitude, _ds.pressure_fl, _ds.sea_salt.where(_ds.sea_salt > 1e-12).fillna(1e-12),
                     dict(norm=LogNorm(1e-12, 1e-6), cmap='Blues'),
                     cbar_kwargs={'pad':0.01, 'label':'mixing ratio\n[kg kg$^{-1}$]', 'ticks':[1e-12, 1e-9, 1e-6]})
    axes[0].set_title('Sea salt')
    axes[0].set_xlabel('')

    irregular_pcolor(axes[1], _ds.latitude, _ds.pressure_fl, _ds.dust.where(_ds.dust > 1e-12).fillna(1e-12),
                     dict(norm=LogNorm(1e-12, 1e-6), cmap='OrRd'),
                     cbar_kwargs={'pad':0.01, 'label':'mixing ratio\n[kg kg$^{-1}$]', 'ticks':[1e-12, 1e-9, 1e-6]})
    axes[1].set_title('Dust')
    axes[1].set_xlabel('')

    irregular_pcolor(axes[2], _ds.latitude, _ds.pressure_fl, _ds.organics.where(_ds.organics > 1e-12).fillna(1e-12),
                     dict(norm=LogNorm(1e-12, 1e-7), cmap='Greens'),
                     cbar_kwargs={'pad':0.01, 'label':'mixing ratio\n[kg kg$^{-1}$]', 'ticks':[1e-12, 1e-10, 1e-8]})
    axes[2].set_title('Organics')
    format_latitude(axes[-1])
    axes[2].set_xlabel('')

    irregular_pcolor(axes[3], _ds.latitude, _ds.pressure_fl, _ds.black_carbon.where(_ds.black_carbon > 1e-12).fillna(1e-12),
                     dict(norm=LogNorm(1e-12, 1e-7), cmap='Greys'),
                     cbar_kwargs={'pad':0.01, 'label':'mixing ratio\n[kg kg$^{-1}$]', 'ticks':[1e-12, 1e-10, 1e-8]})
    axes[3].set_title('Black carbon')
    axes[3].set_xlabel('')

    irregular_pcolor(axes[4], _ds.latitude, _ds.pressure_fl, _ds.sulphate.where(_ds.sulphate > 1e-12).fillna(1e-12),
                     dict(norm=LogNorm(1e-12, 1e-7), cmap='Reds'),
                     cbar_kwargs={'pad':0.01, 'label':'mixing ratio\n[kg kg$^{-1}$]', 'ticks':[1e-12, 1e-10, 1e-8]})
    axes[4].set_title('Sulphates')

    for ax in axes:
        add_temperature_contours(ax, _ds)
        format_pressure(ax)

    axes[-1].set_xlim(-90,90)
    axes[-1].set_xticks(np.arange(-90,91,15))
    axes[-1].set_xlabel('Latitude')

    axes[0].set_yticks([1000e2,800e2,600e2,400e2,200e2,1])
    axes[0].set_ylim(1050e2,1)

    add_subfigure_labels(axes)

    if hasattr(_ds, 'experiment'):
        fig.suptitle(_ds.attrs['experiment'] + "\naerosols", x=get_figure_center(axes[0]), y=0.95, va='top')
    else:
        fig.suptitle("aerosols", x=get_figure_center(axes[0]), y=0.95, va='top')

    if dstfile:
        fig.savefig(dstfile, dpi=90, bbox_inches='tight')
    else:
        return fig, axes

def plot_inputs(IFS_srcfile, dstfile=None, line_ds='default'):
    with sns.plotting_context('notebook', font_scale=1.1), sns.axes_style('ticks'):

        _ds = load_inputs(IFS_srcfile)

        #Set up figure and axes
        nrows=6
        ncols=2

        cbar_kwargs = {'pad':0.0125, 'aspect':10}

        fig, axes= plt.subplots(figsize=(25,2.25*nrows), nrows=nrows, ncols=ncols, gridspec_kw={'wspace':0.0, 'hspace':0.25})

        #First row
        j=0

        ###ATMOSPHERE AND RADIATION
        #First panel SW & surface fields
        i=0
        _ds.cos_solar_zenith_angle.where(_ds.cos_solar_zenith_angle >= 0).plot(ax=axes[i,j], x='latitude', color='k', lw=3)
        axes[i,j].set_xlabel('')
        axes[i,j].set_ylabel(r'$\cos \theta_s$ [-]')
        axes[i,j].set_yticks([0,0.5,1.])
        axes[i,j].set_title('Solar zenith angle and shortwave albedo')
        if 'solar_irradiance' in _ds:
            axes[i,j].text(0.001, 1.01, f"Solar irradiance\n$Q={_ds.solar_irradiance.values:5.1f}$ W m$^{{-2}}$", ha='left', va='bottom', fontsize='small', transform=axes[i,j].transAxes)

        _ax0 = axes[i,j].twinx()
        _ax0.yaxis.set_label_position("right")
        _ax0.yaxis.tick_right()
        if hasattr(_ds, 'sw_albedo_band'):
            _ds.sw_albedo.isel(sw_albedo_band=2).plot.step(ax=_ax0, x='latitude', color=sns.color_palette()[0], lw=4, drawstyle=line_ds)
        else:
            _ds.sw_albedo.plot(ax=_ax0, x='latitude', color=sns.color_palette()[0], lw=4, drawstyle=line_ds)
        _ax0.set_yticks([0,0.5,1.0])
        _ax0.set_yticklabels([0,0.5,1.0],  color=sns.color_palette()[0])
        _ax0.set_ylabel(r'$\alpha_{SW}$ [-]', color=sns.color_palette()[0])

        #Second panel: LW surface fields
        i+=1
        _ds.skin_temperature.plot(ax=axes[i,j], x='latitude', color='k', lw=3)
        axes[i,j].set_xlabel('')
        axes[i,j].set_ylabel(r'$T_s$ [K]')
        axes[i,j].set_title('Skin temperature and longwave emissivity')

        _ax1 = axes[i,j].twinx()
        _ax1.yaxis.set_label_position("right")
        _ax1.yaxis.tick_right()
        if hasattr(_ds, 'lw_emissivity_band'):
            _ds.lw_emissivity.isel(lw_emissivity_band=1).plot.step(ax=_ax1, x='latitude', color=sns.color_palette()[3], lw=4, drawstyle=line_ds)
        else:
            _ds.lw_emissivity.plot(ax=_ax1, x='latitude', color=sns.color_palette()[3], lw=4, drawstyle=line_ds)
        _ax1.set_yticks([0.9,0.95,1.0])
        _ax1.set_ylim(0.89,1.0)
        _ax1.set_yticklabels([0.9,0.95,1.0],  color=sns.color_palette()[3])
        _ax1.set_ylabel(r'$\epsilon_{LW}$ [-]', color=sns.color_palette()[3])

        #Specific humidity
        i+=1
        irregular_pcolor(axes[i,j], _ds.latitude, _ds.pressure_fl, _ds.q,
                         dict(norm=LogNorm(1e-6, 1e-2), cmap='Greens'),
                         cbar_kwargs={**cbar_kwargs, **{'label':'mixing ratio\n[kg kg$^{-1}$]', 'ticks':[1e-5,1e-4,1e-3,1e-2]}})

        axes[i,j].set_title('Specific humidity')
        axes[i,j].set_xlabel('')

        axes[i,j].set_yscale('linear')
        axes[i,j].set_yticks([1000e2,800e2,600e2,400e2,200e2,1])
        axes[i,j].set_ylim(1050e2,1)

        ### CLOUD
        i+=1
        irregular_pcolor(axes[i,j], _ds.latitude, _ds.pressure_fl, _ds.cloud_fraction,
                             dict(vmin=0, vmax=1, cmap='gray_r'),
                             cbar_kwargs={**cbar_kwargs, **{'label':'fraction [-]', 'ticks':[0, 0.2, 0.4, 0.6, 0.8, 1.0]}})
        axes[i,j].set_title('Cloud fraction')
        axes[i,j].set_xlabel('')

        i+=1
        irregular_pcolor(axes[i,j], _ds.latitude, _ds.pressure_fl, _ds.q_ice.where(_ds.q_ice > 1e-10).fillna(1e-10),
                             dict(norm=LogNorm(1e-8, 0.5e-2), cmap='Blues'),
                             cbar_kwargs={**cbar_kwargs, **{'label':'mixing ratio\n[kg kg$^{-1}$]', 'ticks':[1e-7, 1e-5, 1e-3]}})
        axes[i,j].set_title('Cloud ice water content')
        axes[i,j].set_xlabel('')

        i+=1
        irregular_pcolor(axes[i,j], _ds.latitude, _ds.pressure_fl, _ds.q_liquid.where(_ds.q_liquid > 1e-10).fillna(1e-10),
                             dict(norm=LogNorm(1e-8, 0.5e-2), cmap='Reds'),
                             cbar_kwargs={**cbar_kwargs, **{'label':'mixing ratio\n[kg kg$^{-1}$]', 'ticks':[1e-7, 1e-5, 1e-3]}})
        axes[i,j].set_title('Cloud liquid water content')

        ####SECOND COLUMN
        j+=1

        #Ozone
        i=0
        irregular_pcolor(axes[i,j], _ds.latitude, _ds.pressure_fl, _ds.o3_mmr,
                         dict(norm=LogNorm(1e-8, 1e-5), cmap='Blues'),
                         cbar_kwargs={**cbar_kwargs, **{'label':'mixing ratio\n[kg kg$^{-1}$]', 'ticks':[1e-8,1e-7,1e-6,1e-5]}})

        axes[i,j].set_title('Ozone')
        axes[i,j].set_xlabel('')

        axes[i,j].set_yscale('log')
        axes[i,j].set_yticks([1e5,1e4,1e3,1e2,1e1,1e0])
        axes[i,j].set_ylim(1.1e5,1)

        #Aerosols

        i+=1
        irregular_pcolor(axes[i,j], _ds.latitude, _ds.pressure_fl, _ds.sea_salt.where(_ds.sea_salt > 1e-12).fillna(1e-12),
                             dict(norm=LogNorm(1e-12, 1e-6), cmap='Blues'),
                             cbar_kwargs={**cbar_kwargs, **{'label':'mixing ratio\n[kg kg$^{-1}$]', 'ticks':[1e-12, 1e-9, 1e-6]}})
        axes[i,j].set_title('Sea salt')
        axes[i,j].set_xlabel('')

        i+=1
        irregular_pcolor(axes[i,j], _ds.latitude, _ds.pressure_fl, _ds.dust.where(_ds.dust > 1e-12).fillna(1e-12),
                             dict(norm=LogNorm(1e-12, 1e-6), cmap='OrRd'),
                             cbar_kwargs={**cbar_kwargs, **{'label':'mixing ratio\n[kg kg$^{-1}$]', 'ticks':[1e-12, 1e-9, 1e-6]}})
        axes[i,j].set_title('Dust')
        axes[i,j].set_xlabel('')

        i+=1
        irregular_pcolor(axes[i,j], _ds.latitude, _ds.pressure_fl, _ds.organics.where(_ds.organics > 1e-12).fillna(1e-12),
                             dict(norm=LogNorm(1e-12, 1e-7), cmap='Greens'),
                             cbar_kwargs={**cbar_kwargs, **{'label':'mixing ratio\n[kg kg$^{-1}$]', 'ticks':[1e-12, 1e-10, 1e-8]}})
        axes[i,j].set_title('Organics')
        axes[i,j].set_xlabel('')

        i+=1
        irregular_pcolor(axes[i,j], _ds.latitude, _ds.pressure_fl, _ds.black_carbon.where(_ds.black_carbon > 1e-12).fillna(1e-12),
                             dict(norm=LogNorm(1e-12, 1e-7), cmap='Greys'),
                             cbar_kwargs={**cbar_kwargs, **{'label':'mixing ratio\n[kg kg$^{-1}$]', 'ticks':[1e-12, 1e-10, 1e-8]}})
        axes[i,j].set_title('Black carbon')
        axes[i,j].set_xlabel('')

        i+=1
        irregular_pcolor(axes[i,j], _ds.latitude, _ds.pressure_fl, _ds.sulphate.where(_ds.sulphate > 1e-12).fillna(1e-12),
                             dict(norm=LogNorm(1e-12, 1e-7), cmap='Reds'),
                             cbar_kwargs={**cbar_kwargs, **{'label':'mixing ratio\n[kg kg$^{-1}$]', 'ticks':[1e-12, 1e-10, 1e-8]}})
        axes[i,j].set_title('Sulphates')

        for ax in axes[2:,0]:
            add_temperature_contours(ax, _ds)
            format_pressure(ax)
            ax.set_yticks([1000e2,800e2,600e2,400e2,200e2,1])
            ax.set_ylim(1050e2,1)

        for ax in axes[:2,0]:
            snap_to_axis(ax, axes[-1,0])

        for ax in axes[1:,1]:
            add_temperature_contours(ax, _ds)
            ax.set_yticks([1000e2,800e2,600e2,400e2,200e2,1])
            ax.set_ylim(1050e2,1)
            ax.set_yticklabels([])
            ax.set_ylabel("")

        format_pressure(axes[0,1])
        add_temperature_contours(axes[0,1], _ds)

        for ax in axes.flatten():
            ax.set_xlim(-90,90)
            ax.set_xticks(np.arange(-90,91,15))
            ax.set_xlabel('')
            ax.set_xticklabels([])

        for ax in axes[-1,:].flatten():
            format_latitude(ax)
            ax.set_xlabel('Latitude')

        import string
        add_subfigure_labels(axes)

        name_string = os.path.splitext(os.path.basename(IFS_srcfile))[0]

        x = (get_figure_center(axes[0,0]) + get_figure_center(axes[0,1]))/2
        y = get_figure_top(fig, axes[0,0], include_hspace=True)

        #fig.suptitle(f"{name_string}\nIFS cloud, aerosol and radiation fields", x=x, y=y-0.025, va='top', fontsize=30)

        fig.suptitle(f"{name_string}\nIFS cloud, aerosol and radiation fields", x=x, y=y-0.07, va='bottom', fontsize=25)

        if dstfile:
            fig.savefig(dstfile, dpi=90, bbox_inches='tight')
        else:
            return fig, axes

def plot_LW_flux(IFS_srcfile, ecRAD_srcfile, dstfile=None, clearsky=False):
    _ds = load_ecRAD(ecRAD_srcfile, IFS_srcfile)

    # LW fluxes
    nrows=3
    fig, axes = plt.subplots(figsize=(25,nrows*4), nrows=nrows, sharex=True, sharey=True)

    if clearsky:
        irregular_pcolor(axes[0], _ds.latitude, _ds.pressure_hl, _ds.flux_dn_lw_clear,
                         dict(cmap='Reds', vmin=0, vmax=500),
                         cbar_kwargs={'pad':0.01, 'label':'flux [W m$^{-2}$]'})
        axes[0].set_title("Clear-sky downwelling")
    else:
        irregular_pcolor(axes[0], _ds.latitude, _ds.pressure_hl, _ds.flux_dn_lw,
                         dict(cmap='Reds', vmin=0, vmax=500),
                         cbar_kwargs={'pad':0.01, 'label':'flux [W m$^{-2}$]'})
        axes[0].set_title("Downwelling")
    axes[0].set_xlabel('')

    if clearsky:
        irregular_pcolor(axes[1], _ds.latitude, _ds.pressure_hl, _ds.flux_up_lw_clear,
                         dict(cmap='Reds', vmin=0, vmax=500),
                         cbar_kwargs={'pad':0.01, 'label':'flux [W m$^{-2}$]'})
        axes[1].set_title("Clear-sky upwelling")
    else:
        irregular_pcolor(axes[1], _ds.latitude, _ds.pressure_hl, _ds.flux_up_lw,
                         dict(cmap='Reds', vmin=0, vmax=500),
                         cbar_kwargs={'pad':0.01, 'label':'flux [W m$^{-2}$]'})
        axes[1].set_title("Upwelling")
    axes[1].set_xlabel('')

    if clearsky:
        irregular_pcolor(axes[2], _ds.latitude, _ds.pressure_hl, _ds.flux_net_lw_clear,
                         dict(cmap='RdBu_r', norm=DivergingNorm(vcenter=0)),#, center=0, robust=True),
                         cbar_kwargs={'pad':0.01, 'label':'flux [W m$^{-2}$]'})
        axes[2].set_title("Clear-sky net")
    else:
        irregular_pcolor(axes[2], _ds.latitude, _ds.pressure_hl, _ds.flux_net_lw,
                         dict(cmap='RdBu_r', norm=DivergingNorm(vcenter=0)),#, center=0, robust=True),
                         cbar_kwargs={'pad':0.01, 'label':'flux [W m$^{-2}$]'})
        axes[2].set_title("Net")

    for ax in axes:
        add_temperature_contours(ax, _ds)
        format_pressure(ax)

    for ax in axes:
        if True:
            ax.set_yscale('linear')
            ax.set_yticks([1000e2,800e2,600e2,400e2,200e2,1])
            ax.set_ylim(1050e2,1)
        else:
            ax.set_yscale('log')
            ax.set_yticks([1e5,1e4,1e3,1e2,1e1,1e0])
            ax.set_ylim(1.1e5,1)
        format_pressure(ax)

    axes[-1].set_xlim(-90,90)
    axes[-1].set_xticks(np.arange(-90,91,15))
    format_latitude(axes[-1])
    axes[-1].set_xlabel('Latitude')

    if hasattr(_ds, 'experiment'):
        place_suptitle(fig, axes, _ds.attrs['experiment'] + "\nLongwave fluxes", y=1.0)
    else:
        place_suptitle(fig, axes, "Longwave fluxes")

    add_subfigure_labels(axes)

    if dstfile:
        fig.savefig(dstfile, dpi=90, bbox_inches='tight')
    else:
        return fig, axes

def plot_LW_flux_difference(IFS_srcfile, ecRAD_srcfile, reference_ecRAD_srcfile, title=None, dstfile=None, clearsky=False):
    ds = load_ecRAD(reference_ecRAD_srcfile, IFS_srcfile)
    _ds = load_ecRAD(ecRAD_srcfile, IFS_srcfile)

    # LW fluxes
    nrows=3
    fig, axes = plt.subplots(figsize=(25,nrows*4), nrows=nrows, sharex=True, sharey=True)

    cbar_kwargs = {'pad':0.01, 'label':'$\Delta$ flux [W m$^{-2}$]'}

    if clearsky:
        da = (_ds.flux_dn_lw_clear - ds.flux_dn_lw_clear)
        vmin, vmax = get_vextents(da)
        irregular_pcolor(axes[0], _ds.latitude, _ds.pressure_hl, da,
                         dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                         cbar_kwargs=cbar_kwargs)
        axes[0].set_title("Clear-sky downwelling")
    else:
        da = (_ds.flux_dn_lw - ds.flux_dn_lw)
        vmin, vmax = get_vextents(da)
        irregular_pcolor(axes[0], _ds.latitude, _ds.pressure_hl, da,
                         dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                         cbar_kwargs=cbar_kwargs)
        axes[0].set_title("Downwelling")
    axes[0].set_xlabel('')

    if clearsky:
        da = (_ds.flux_up_lw_clear - ds.flux_up_lw_clear)
        vmin, vmax = get_vextents(da)
        irregular_pcolor(axes[1], _ds.latitude, _ds.pressure_hl, da,
                         dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                         cbar_kwargs=cbar_kwargs)
        axes[1].set_title("Clear-sky upwelling")
    else:
        da = (_ds.flux_up_lw - ds.flux_up_lw)
        vmin, vmax = get_vextents(da)
        irregular_pcolor(axes[1], _ds.latitude, _ds.pressure_hl, da,
                         dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                              cbar_kwargs=cbar_kwargs)
        axes[1].set_title("Upwelling")
    axes[1].set_xlabel('')

    if clearsky:
        da = (_ds.flux_net_lw_clear - ds.flux_net_lw_clear)
        vmin, vmax = get_vextents(da)
        irregular_pcolor(axes[2], _ds.latitude, _ds.pressure_hl, da,
                         dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                         cbar_kwargs=cbar_kwargs)
        axes[2].set_title("Clear-sky net")
    else:
        da = (_ds.flux_net_lw - ds.flux_net_lw)
        vmin, vmax = get_vextents(da)
        irregular_pcolor(axes[2], _ds.latitude, _ds.pressure_hl, da,
                         dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                              cbar_kwargs=cbar_kwargs)
        axes[2].set_title("Net ")

    for ax in axes:
        add_temperature_contours(ax, _ds)
        format_pressure(ax)

    for ax in axes:
        if True:
            ax.set_yscale('linear')
            ax.set_yticks([1000e2,800e2,600e2,400e2,200e2,1])
            ax.set_ylim(1050e2,1)
        else:
            ax.set_yscale('log')
            ax.set_yticks([1e5,1e4,1e3,1e2,1e1,1e0])
            ax.set_ylim(1.1e5,1)
        format_pressure(ax)

    axes[-1].set_xlim(-90,90)
    axes[-1].set_xticks(np.arange(-90,91,15))
    format_latitude(axes[-1])
    axes[-1].set_xlabel('Latitude')

    if hasattr(_ds, 'experiment'):
        place_suptitle(fig, axes, f"{_ds.attrs['experiment']} $-$ {ds.attrs['experiment']}\nLongwave fluxes", y=1.0)
    else:
        place_suptitle(fig, axes, "Longwave fluxes")

    add_subfigure_labels(axes)

    if dstfile:
        fig.savefig(dstfile, dpi=90, bbox_inches='tight')
    else:
        return fig, axes

def plot_SW_flux(IFS_srcfile, ecRAD_srcfile, title=None, dstfile=None, clearsky=False):
    _ds = load_ecRAD(ecRAD_srcfile, IFS_srcfile)

    # SW fluxes
    nrows=3
    fig, axes = plt.subplots(figsize=(25,nrows*4), nrows=nrows, sharex=True, sharey=True)

    cbar_kwargs = {'pad':0.01, 'label':'flux [W m$^{-2}$]'}

    if clearsky:
        irregular_pcolor(axes[0], _ds.latitude, _ds.pressure_hl, _ds.flux_dn_sw_clear,
                         dict(cmap='Blues', vmin=0),
                         cbar_kwargs=cbar_kwargs)
        axes[0].set_title("Clear-sky downwelling")
    else:
        irregular_pcolor(axes[0], _ds.latitude, _ds.pressure_hl, _ds.flux_dn_sw,
                         dict(cmap='Blues', vmin=0),
                         cbar_kwargs=cbar_kwargs)
        axes[0].set_title("Downwelling")
    axes[0].set_xlabel('')

    if clearsky:
        irregular_pcolor(axes[1], _ds.latitude, _ds.pressure_hl, _ds.flux_up_sw_clear,
                         dict(cmap='Blues', vmin=0),
                         cbar_kwargs=cbar_kwargs)
        axes[1].set_title("Clear-sky upwelling")
    else:
        irregular_pcolor(axes[1], _ds.latitude, _ds.pressure_hl, _ds.flux_up_sw,
                         dict(cmap='Blues', vmin=0),
                         cbar_kwargs=cbar_kwargs)
        axes[1].set_title("Upwelling")
    axes[1].set_xlabel('')

    if clearsky:
        if (_ds.flux_net_sw_clear.quantile(q=0.01) < 0) & (0 < _ds.flux_net_sw_clear.quantile(q=0.99)):
            irregular_pcolor(axes[2], _ds.latitude, _ds.pressure_hl, _ds.flux_net_sw_clear,
                             dict(cmap='RdBu_r', norm=DivergingNorm(vcenter=0, vmin=_ds.flux_net_sw_clear.quantile(q=0.01), vmax=_ds.flux_net_sw_clear.quantile(q=0.99))),
                             cbar_kwargs=cbar_kwargs)
        else:
            irregular_pcolor(axes[2], _ds.latitude, _ds.pressure_hl, _ds.flux_net_sw_clear,
                             dict(cmap='Blues_r', vmax=0),
                             cbar_kwargs=cbar_kwargs)
        axes[2].set_title("Clear-sky net")
    else:
        if (_ds.flux_net_sw_clear.quantile(q=0.01) < 0) & (0 < _ds.flux_net_sw_clear.quantile(q=0.99)):
            irregular_pcolor(axes[2], _ds.latitude, _ds.pressure_hl, _ds.flux_net_sw,
                             dict(cmap='RdBu_r', norm=DivergingNorm(vcenter=0, vmin=_ds.flux_net_sw.quantile(q=0.01), vmax=_ds.flux_net_sw.quantile(q=0.99))),
                             cbar_kwargs=cbar_kwargs)
        else:
            irregular_pcolor(axes[2], _ds.latitude, _ds.pressure_hl, _ds.flux_net_sw,
                             dict(cmap='Blues_r', vmax=0),
                             cbar_kwargs=cbar_kwargs)
        axes[2].set_title("Net")

    for ax in axes:
        add_temperature_contours(ax, _ds)
        format_pressure(ax)

    for ax in axes:
        if True:
            ax.set_yscale('linear')
            ax.set_yticks([1000e2,800e2,600e2,400e2,200e2,1])
            ax.set_ylim(1050e2,1)
        else:
            ax.set_yscale('log')
            ax.set_yticks([1e5,1e4,1e3,1e2,1e1,1e0])
            ax.set_ylim(1.1e5,1)
        format_pressure(ax)

    axes[-1].set_xlim(-90,90)
    axes[-1].set_xticks(np.arange(-90,91,15))
    format_latitude(axes[-1])
    axes[-1].set_xlabel('Latitude')

    if hasattr(_ds, 'experiment'):
        place_suptitle(fig, axes, _ds.attrs['experiment'] + "\nShortwave fluxes", y=1.0)
    else:
        place_suptitle(fig, axes, "Shortwave fluxes")

    add_subfigure_labels(axes)

    if dstfile:
        fig.savefig(dstfile, dpi=90, bbox_inches='tight')
    else:
        return fig, axes


def plot_SW_flux_difference(IFS_srcfile, ecRAD_srcfile, reference_ecRAD_srcfile, title=None, dstfile=None, clearsky=False):
    ds = load_ecRAD(reference_ecRAD_srcfile, IFS_srcfile).load()
    _ds = load_ecRAD(ecRAD_srcfile, IFS_srcfile).load()

    cbar_kwargs = {'pad':0.01, 'label':'$\Delta$ flux [W m$^{-2}$]'}

    nrows=3
    fig, axes = plt.subplots(figsize=(25,nrows*4), nrows=nrows, sharex=True, sharey=True)

    if clearsky:
        da = (_ds.flux_dn_sw_clear - ds.flux_dn_sw_clear)
        vmin, vmax = get_vextents(da)
        irregular_pcolor(axes[0], _ds.latitude, _ds.pressure_hl, da,
                         dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                         cbar_kwargs=cbar_kwargs)
        axes[0].set_title("Clear-sky downwelling")

    else:
        da = (_ds.flux_dn_sw - ds.flux_dn_sw)
        vmin, vmax = get_vextents(da)
        irregular_pcolor(axes[0], _ds.latitude, _ds.pressure_hl, da,
                         dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                         cbar_kwargs=cbar_kwargs)
        axes[0].set_title("Downwelling")
    axes[0].set_xlabel('')

    if clearsky:
        da = (_ds.flux_up_sw_clear - ds.flux_up_sw_clear)
        vmin, vmax = get_vextents(da)
        irregular_pcolor(axes[1], _ds.latitude, _ds.pressure_hl, da,
                         dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                         cbar_kwargs=cbar_kwargs)

        axes[1].set_title("Clear-sky upwelling")

    else:
        da = (_ds.flux_up_sw - ds.flux_up_sw)
        vmin, vmax = get_vextents(da)
        irregular_pcolor(axes[1], _ds.latitude, _ds.pressure_hl, da,
                         dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                         cbar_kwargs=cbar_kwargs)
        axes[1].set_title("Upwelling")
    axes[1].set_xlabel('')

    if clearsky:
        da = (_ds.flux_net_sw_clear - ds.flux_net_sw_clear)
        vmin, vmax = get_vextents(da)
        irregular_pcolor(axes[2], _ds.latitude, _ds.pressure_hl, da,
                         dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                         cbar_kwargs=cbar_kwargs)
        axes[2].set_title("Clear-sky net")

    else:
        da = (_ds.flux_net_sw - ds.flux_net_sw)
        vmin, vmax = get_vextents(da)
        irregular_pcolor(axes[2], _ds.latitude, _ds.pressure_hl, da,
                         dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                         cbar_kwargs=cbar_kwargs)
        axes[2].set_title("Net")

    for ax in axes:
        add_temperature_contours(ax, _ds)
        format_pressure(ax)

    for ax in axes:
        if True:
            ax.set_yscale('linear')
            ax.set_yticks([1000e2,800e2,600e2,400e2,200e2,1])
            ax.set_ylim(1050e2,1)
        else:
            ax.set_yscale('log')
            ax.set_yticks([1e5,1e4,1e3,1e2,1e1,1e0])
            ax.set_ylim(1.1e5,1)
        format_pressure(ax)

    axes[-1].set_xlim(-90,90)
    axes[-1].set_xticks(np.arange(-90,91,15))
    format_latitude(axes[-1])
    axes[-1].set_xlabel('Latitude')

    if hasattr(_ds, 'experiment'):
        place_suptitle(fig, axes, f"{_ds.attrs['experiment']} $-$ {ds.attrs['experiment']}\nShortwave fluxes", y=1.0)
    else:
        place_suptitle(fig, axes, "Shortwave fluxes")

    add_subfigure_labels(axes)

    if dstfile:
        fig.savefig(dstfile, dpi=90, bbox_inches='tight')
    else:
        return fig, axes


def plot_CRE(IFS_srcfile, ecRAD_srcfile, title=None, dstfile=None):
    _ds = load_ecRAD(ecRAD_srcfile, IFS_srcfile)

    nrows=3
    fig, axes = plt.subplots(figsize=(25,nrows*4), nrows=nrows, sharex=True, sharey=True)

    da = _ds.cloud_radiative_effect_sw
    vmin, vmax = get_vextents(da)
    irregular_pcolor(axes[0], _ds.latitude, _ds.pressure_hl, da,
                         dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                         cbar_kwargs={'pad':0.01, 'label':'CRE$_{\mathrm{SW}}$ [W m$^{-2}$]'})

    axes[0].set_xlabel('')
    axes[0].set_title("Shortwave")

    da = _ds.cloud_radiative_effect_lw
    vmin, vmax= get_vextents(da)
    irregular_pcolor(axes[1], _ds.latitude, _ds.pressure_hl, da,
                         dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                         cbar_kwargs={'pad':0.01, 'label':'CRE$_{\mathrm{LW}}$ [W m$^{-2}$]'})
    axes[1].set_xlabel('')
    axes[1].set_title("Longwave")

    da = (_ds.cloud_radiative_effect_sw + _ds.cloud_radiative_effect_lw)
    vmin, vmax= get_vextents(da)
    irregular_pcolor(axes[2], _ds.latitude, _ds.pressure_hl, da,
                     dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                     cbar_kwargs={'pad':0.01, 'label':'CRE$_{\mathrm{net}}$ [W m$^{-2}$]'})
    axes[2].set_title("Net")

    for ax in axes:
        add_temperature_contours(ax, _ds)
        format_pressure(ax)

    for ax in axes:
        if True:
            ax.set_yscale('linear')
            ax.set_yticks([1000e2,800e2,600e2,400e2,200e2,1])
            ax.set_ylim(1050e2,1)
        else:
            ax.set_yscale('log')
            ax.set_yticks([1e5,1e4,1e3,1e2,1e1,1e0])
            ax.set_ylim(1.1e5,1)
        format_pressure(ax)

    axes[-1].set_xlim(-90,90)
    axes[-1].set_xticks(np.arange(-90,91,15))
    format_latitude(axes[-1])
    axes[-1].set_xlabel('Latitude')

    if hasattr(_ds, 'experiment'):
        place_suptitle(fig, axes, _ds.attrs['experiment'] + "\nCloud radiative effects", y=1.0)
    else:
        place_suptitle(fig, axes, "Cloud radiative effects")

    add_subfigure_labels(axes)

    if dstfile:
        fig.savefig(dstfile, dpi=90, bbox_inches='tight')
    else:
        return fig, axes


def plot_CRE_difference(IFS_srcfile, ecRAD_srcfile, reference_ecRAD_srcfile, title=None, dstfile=None):
    name_string = os.path.splitext(os.path.basename(IFS_srcfile))[0]

    ds = load_ecRAD(reference_ecRAD_srcfile, IFS_srcfile)
    _ds = load_ecRAD(ecRAD_srcfile, IFS_srcfile)

    nrows=3
    fig, axes = plt.subplots(figsize=(25,nrows*4), nrows=nrows, sharex=True, sharey=True)

    da = (_ds.cloud_radiative_effect_sw - ds.cloud_radiative_effect_sw)
    vmin, vmax= get_vextents(da)
    irregular_pcolor(axes[0], _ds.latitude, _ds.pressure_hl, da,
                     dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                     cbar_kwargs={'pad':0.01, 'label':'$\Delta$ CRE$_{\mathrm{SW}}$ [W m$^{-2}$]'})
    axes[0].set_xlabel('')
    axes[0].set_title("Shortwave")

    da = (_ds.cloud_radiative_effect_lw - ds.cloud_radiative_effect_lw)
    vmin, vmax= get_vextents(da)
    irregular_pcolor(axes[1], _ds.latitude, _ds.pressure_hl, da,
                     dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                     cbar_kwargs={'pad':0.01, 'label':'$\Delta$ CRE$_{\mathrm{SW}}$ [W m$^{-2}$]'})
    axes[1].set_xlabel('')
    axes[1].set_title("Longwave")

    da = ((_ds.cloud_radiative_effect_sw + _ds.cloud_radiative_effect_lw) - (ds.cloud_radiative_effect_sw + ds.cloud_radiative_effect_lw))
    vmin, vmax= get_vextents(da)
    irregular_pcolor(axes[2], _ds.latitude, _ds.pressure_hl, da,
                     dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                     cbar_kwargs={'pad':0.01, 'label':'$\Delta$ CRE$_\mathrm{net}$ [W m$^{-2}$]'})
    axes[2].set_title("Net")

    for ax in axes:
        add_temperature_contours(ax, _ds)
        format_pressure(ax)

    for ax in axes:
        if True:
            ax.set_yscale('linear')
            ax.set_yticks([1000e2,800e2,600e2,400e2,200e2,1])
            ax.set_ylim(1050e2,1)
        else:
            ax.set_yscale('log')
            ax.set_yticks([1e5,1e4,1e3,1e2,1e1,1e0])
            ax.set_ylim(1.1e5,1)
        format_pressure(ax)

    axes[-1].set_xlim(-90,90)
    axes[-1].set_xticks(np.arange(-90,91,15))
    format_latitude(axes[-1])
    axes[-1].set_xlabel('Latitude')

    if hasattr(_ds, 'experiment'):
        place_suptitle(fig, axes, f"{name_string}\n{_ds.attrs['experiment']} $-$ {ds.attrs['experiment']}\nCloud radiative effects", y=1.0)
    else:
        place_suptitle(fig, axes, "Cloud radiative effects")

    add_subfigure_labels(axes)

    if dstfile:
        fig.savefig(dstfile, dpi=90, bbox_inches='tight')
    else:
        return fig, axes


def plot_heating_rate(IFS_srcfile, ecRAD_srcfile, title=None, linear_pressure=True, dstfile=None):
    name_string = os.path.splitext(os.path.basename(IFS_srcfile))[0]

    _ds = load_ecRAD(ecRAD_srcfile, IFS_srcfile)

    if linear_pressure:
        vmax = 10
    else:
        vmax = 30

    nrows=3
    fig, axes = plt.subplots(figsize=(25,nrows*4), nrows=nrows, sharex=True, sharey=True)

    cbar_kwargs = {'pad':0.01, 'label':'$\dfrac{dT}{dt}$ [K d$^{-1}$]'}

    da = _ds.heating_rate_lw
    if linear_pressure:
        vmin, vmax= get_vextents(da.where(_ds.pressure_fl > 100e2))
    else:
        vmin, vmax= get_vextents(da)
    irregular_pcolor(axes[0], _ds.latitude, _ds.pressure_fl, da,
                     dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                     cbar_kwargs=cbar_kwargs)
    axes[0].set_xlabel('')
    axes[0].set_title("Longwave")

    da = _ds.heating_rate_sw
    if linear_pressure:
        vmin, vmax= get_vextents(da.where(_ds.pressure_fl > 100e2))
    else:
        vmin, vmax= get_vextents(da)
    irregular_pcolor(axes[1], _ds.latitude, _ds.pressure_fl, da,
                     dict(cmap='RdBu_r',vmin=vmin, vmax=vmax),
                     cbar_kwargs=cbar_kwargs)
    axes[1].set_xlabel('')
    axes[1].set_title("Shortwave")

    da = (_ds.heating_rate_sw + _ds.heating_rate_lw)
    if linear_pressure:
        vmin, vmax= get_vextents(da.where(_ds.pressure_fl > 100e2))
    else:
        vmin, vmax= get_vextents(da)
    irregular_pcolor(axes[2], _ds.latitude, _ds.pressure_fl, da,
                     dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                     cbar_kwargs=cbar_kwargs)
    axes[2].set_title("Net")

    for ax in axes:
        add_temperature_contours(ax, _ds)
        format_pressure(ax)

    for ax in axes:
        if linear_pressure:
            ax.set_yscale('linear')
            ax.set_yticks([1000e2,800e2,600e2,400e2,200e2,1])
            ax.set_ylim(1050e2,1)
        else:
            ax.set_yscale('log')
            ax.set_yticks([1e5,1e4,1e3,1e2,1e1,1e0])
            ax.set_ylim(1.1e5,1)
        format_pressure(ax)

    axes[-1].set_xlim(-90,90)
    axes[-1].set_xticks(np.arange(-90,91,15))
    format_latitude(axes[-1])
    axes[-1].set_xlabel('Latitude')

    if hasattr(_ds, 'experiment'):
        place_suptitle(fig, axes, f"{name_string}\n{_ds.attrs['experiment']}\nHeating rates", y=1.0)
    else:
        place_suptitle(fig, axes, "Heating rates")

    add_subfigure_labels(axes)

    if dstfile:
        fig.savefig(dstfile, dpi=90, bbox_inches='tight')
    else:
        return fig, axes


def plot_heating_rate_difference(IFS_srcfile, ecRAD_srcfile, reference_ecRAD_srcfile, title=None, linear_pressure=True, dstfile=None):
    name_string = os.path.splitext(os.path.basename(IFS_srcfile))[0]

    ds = load_ecRAD(reference_ecRAD_srcfile, IFS_srcfile)
    _ds = load_ecRAD(ecRAD_srcfile, IFS_srcfile)

    if linear_pressure:
        vmax = 10
    else:
        vmax = 30

    nrows=3
    fig, axes = plt.subplots(figsize=(25,nrows*4), nrows=nrows, sharex=True, sharey=True)

    cbar_kwargs = {'pad':0.01, 'label':'$\Delta \dfrac{dT}{dt}$ [K d$^{-1}$]'}

    da = (_ds.heating_rate_lw - ds.heating_rate_lw)
    if linear_pressure:
        vmin, vmax= get_vextents(da.where(_ds.pressure_fl > 100e2))
    else:
        vmin, vmax= get_vextents(da)
    irregular_pcolor(axes[0], _ds.latitude, _ds.pressure_fl, da,
                     dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                     cbar_kwargs=cbar_kwargs)
    axes[0].set_xlabel('')
    axes[0].set_title("Longwave")

    da = (_ds.heating_rate_sw - ds.heating_rate_sw)
    if linear_pressure:
        vmin, vmax= get_vextents(da.where(_ds.pressure_fl > 100e2))
    else:
        vmin, vmax= get_vextents(da)
    irregular_pcolor(axes[1], _ds.latitude, _ds.pressure_fl, da,
                     dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                     cbar_kwargs=cbar_kwargs)
    axes[1].set_xlabel('')
    axes[1].set_title("Shortwave")

    da = ((_ds.heating_rate_sw + _ds.heating_rate_lw) - (ds.heating_rate_sw + ds.heating_rate_lw))
    if linear_pressure:
        vmin, vmax= get_vextents(da.where(_ds.pressure_fl > 100e2))
    else:
        vmin, vmax= get_vextents(da)
    irregular_pcolor(axes[2], _ds.latitude, _ds.pressure_fl, da,
                     dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                     cbar_kwargs=cbar_kwargs)
    axes[2].set_title("Net")

    for ax in axes:
        add_temperature_contours(ax, _ds)
        format_pressure(ax)

    for ax in axes:
        if linear_pressure:
            ax.set_yscale('linear')
            ax.set_yticks([1000e2,800e2,600e2,400e2,200e2,1])
            ax.set_ylim(1050e2,1)
        else:
            ax.set_yscale('log')
            ax.set_yticks([1e5,1e4,1e3,1e2,1e1,1e0])
            ax.set_ylim(1.1e5,1)
        format_pressure(ax)

    axes[-1].set_xlim(-90,90)
    axes[-1].set_xticks(np.arange(-90,91,15))
    format_latitude(axes[-1])
    axes[-1].set_xlabel('Latitude')

    if hasattr(_ds, 'experiment'):
        place_suptitle(fig, axes, f"{name_string}\n{_ds.attrs['experiment']} $-$ {ds.attrs['experiment']}\nHeating rates", y=1.0)
    else:
        place_suptitle(fig, axes, "Heating rates")

    add_subfigure_labels(axes)

    if dstfile:
        fig.savefig(dstfile, dpi=90, bbox_inches='tight')
    else:
        return fig, axes


def plot_output(IFS_srcfile, ecRAD_srcfile, dstfile=None):

    with sns.plotting_context('notebook', font_scale=1.1), sns.axes_style('ticks'):

        _ds = load_ecRAD(ecRAD_srcfile, IFS_srcfile)

        # LW fluxes
        nrows=5
        ncols=2

        fig, axes = plt.subplots(figsize=(25,2.5*nrows), nrows=nrows, ncols=ncols, sharex=True, sharey='row', gridspec_kw={'hspace':0.25, 'wspace':0.0})

        ### LW fluxes
        j=0
        i=0
        irregular_pcolor(axes[i,j], _ds.latitude, _ds.pressure_hl, _ds.flux_dn_lw,
                             dict(cmap='Reds', vmin=0, vmax=500),
                             cbar_kwargs={'pad':0.0125, 'aspect':10, 'label':'flux [W m$^{-2}$]'})
        axes[i,j].set_title("Downwelling longwave flux")
        axes[i,j].set_xlabel('')

        i+=1
        irregular_pcolor(axes[i,j], _ds.latitude, _ds.pressure_hl, _ds.flux_up_lw,
                             dict(cmap='Reds', vmin=0, vmax=500),
                             cbar_kwargs={'pad':0.0125, 'aspect':10,'label':'flux [W m$^{-2}$]'})
        axes[i,j].set_title("Upwelling longwave flux")
        axes[i,j].set_xlabel('')

        #SW fluxes
        j=1
        i=0
        irregular_pcolor(axes[i,j], _ds.latitude, _ds.pressure_hl, _ds.flux_dn_sw,
                             dict(cmap='Reds', vmin=0, vmax=1300),
                             cbar_kwargs={'pad':0.0125, 'aspect':10, 'label':'flux [W m$^{-2}$]'})
        axes[i,j].set_title("Downwelling shortwave flux")
        axes[i,j].set_xlabel('')

        i+=1
        irregular_pcolor(axes[i,j], _ds.latitude, _ds.pressure_hl, _ds.flux_up_sw,
                             dict(cmap='Reds', vmin=0, vmax=1300),
                             cbar_kwargs={'pad':0.0125, 'aspect':10,'label':'flux [W m$^{-2}$]'})
        axes[i,j].set_title("Upwelling shortwave flux")
        axes[i,j].set_xlabel('')

        #Cloud radiative effects
        j=0
        i+=1
        da = _ds.cloud_radiative_effect_lw
        vmin, vmax = get_vextents(da)
        irregular_pcolor(axes[i,j], _ds.latitude, _ds.pressure_hl, da,
                             dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                             cbar_kwargs={'pad':0.0125, 'aspect':10, 'label':'CRE$_{\mathrm{LW}}$ [W m$^{-2}$]'})

        axes[i,j].set_xlabel('')
        axes[i,j].set_title("Longwave cloud radiative effect")

        j+=1
        da = _ds.cloud_radiative_effect_sw
        vmin, vmax= get_vextents(da)
        irregular_pcolor(axes[i,j], _ds.latitude, _ds.pressure_hl, da,
                             dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                             cbar_kwargs={'pad':0.0125, 'aspect':10, 'label':'CRE$_{\mathrm{SW}}$ [W m$^{-2}$]'})
        axes[i,j].set_xlabel('')
        axes[i,j].set_title("Shortwave cloud radiative effect")

        #Heating rates
        cbar_kwargs = {'pad':0.0125, 'aspect':10, 'label':'$\dfrac{dT}{dt}$ [K d$^{-1}$]'}
        j=0
        i+=1

        linear_pressure=True

        da = _ds.heating_rate_lw
        if linear_pressure:
            vmin, vmax= get_vextents(da.where(_ds.pressure_fl > 100e2))
        else:
            vmin, vmax= get_vextents(da)
        irregular_pcolor(axes[i,j], _ds.latitude, _ds.pressure_fl, da,
                         dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                         cbar_kwargs=cbar_kwargs)
        axes[i,j].set_xlabel('')
        axes[i,j].set_title("Longwave heating rate (troposphere)")

        j+=1
        da = _ds.heating_rate_sw
        if linear_pressure:
            vmin, vmax= get_vextents(da.where(_ds.pressure_fl > 100e2))
        else:
            vmin, vmax= get_vextents(da)
        irregular_pcolor(axes[i,j], _ds.latitude, _ds.pressure_fl, da,
                         dict(cmap='RdBu_r',vmin=vmin, vmax=vmax),
                         cbar_kwargs=cbar_kwargs)
        axes[i,j].set_xlabel('')
        axes[i,j].set_title("Shortwave heating rate (troposphere)")

        j=0
        i+=1

        linear_pressure=False

        da = _ds.heating_rate_lw
        if linear_pressure:
            vmin, vmax= get_vextents(da.where(_ds.pressure_fl > 100e2))
        else:
            vmin, vmax= get_vextents(da)
        irregular_pcolor(axes[i,j], _ds.latitude, _ds.pressure_fl, da,
                         dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                         cbar_kwargs=cbar_kwargs)
        axes[i,j].set_xlabel('')
        axes[i,j].set_title("Longwave heating rate (stratosphere)")

        j+=1
        da = _ds.heating_rate_sw
        if linear_pressure:
            vmin, vmax= get_vextents(da.where(_ds.pressure_fl > 100e2))
        else:
            vmin, vmax= get_vextents(da)
        irregular_pcolor(axes[i,j], _ds.latitude, _ds.pressure_fl, da,
                         dict(cmap='RdBu_r',vmin=vmin, vmax=vmax),
                         cbar_kwargs=cbar_kwargs)
        axes[i,j].set_xlabel('')
        axes[i,j].set_title("Shortwave heating rate (stratosphere)")

        for ax in axes[:-1,:].flatten():
            add_temperature_contours(ax, _ds)
            ax.set_yscale('linear')
            ax.set_yticks([1000e2,800e2,600e2,400e2,200e2,1])
            ax.set_ylim(1050e2,1)

        for ax in axes[-1,:].flatten():
            add_temperature_contours(ax, _ds)
            ax.set_yscale('log')
            ax.set_yticks([1e5,1e4,1e3,1e2,1e1,1e0])
            ax.set_ylim(1.1e5,1)

            ax.set_xlim(-90,90)
            ax.set_xticks(np.arange(-90,91,15))
            format_latitude(ax)
            ax.set_xlabel('Latitude')

        for ax in axes[:,0]:
            format_pressure(ax)

        x = (get_figure_center(axes[0,0]) + get_figure_center(axes[0,1]))/2
        y = get_figure_top(fig, axes[0,0], include_hspace=True)

        name_string = os.path.splitext(os.path.basename(IFS_srcfile))[0]
        fig.suptitle(f"{name_string}\n{_ds.attrs['experiment']}\nFluxes, cloud radiative effects and heating rates", x=x, y=y-0.025, va='bottom', fontsize='xx-large')

        add_subfigure_labels(axes, flatten_order='C')

        if dstfile:
            fig.savefig(dstfile, dpi=90, bbox_inches='tight')
        else:
            return fig, axes


def compare_output(IFS_srcfile, ctrl_srcfile, ecRAD_srcfile, dstfile=None):

    ds = load_ecRAD(ctrl_srcfile, IFS_srcfile)

    with sns.plotting_context('notebook', font_scale=1.1), sns.axes_style('ticks'):

        _ds = load_ecRAD(ecRAD_srcfile, IFS_srcfile)

        # LW fluxes
        nrows=5
        ncols=2

        fig, axes = plt.subplots(figsize=(25,2.5*nrows), nrows=nrows, ncols=ncols, sharex=True, sharey='row', gridspec_kw={'hspace':0.25, 'wspace':0.0})

        ### LW fluxes
        j=0
        i=0
        da = _ds.flux_dn_lw - ds.flux_dn_lw
        vmin, vmax = get_vextents(da, symmetric=True)
        irregular_pcolor(axes[i,j], _ds.latitude, _ds.pressure_hl, da,
                             dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                             cbar_kwargs={'pad':0.0125, 'aspect':10, 'label':'flux [W m$^{-2}$]'})
        axes[i,j].set_title("Change to downwelling longwave flux")
        axes[i,j].set_xlabel('')

        i+=1
        da = _ds.flux_up_lw - ds.flux_up_lw
        vmin, vmax = get_vextents(da, symmetric=True)
        irregular_pcolor(axes[i,j], _ds.latitude, _ds.pressure_hl, da,
                             dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                             cbar_kwargs={'pad':0.0125, 'aspect':10, 'label':'flux [W m$^{-2}$]'})
        axes[i,j].set_title("Change to upwelling longwave flux")
        axes[i,j].set_xlabel('')

        #SW fluxes
        j=1
        i=0
        da = _ds.flux_dn_sw - ds.flux_dn_sw
        vmin, vmax = get_vextents(da, symmetric=True)
        irregular_pcolor(axes[i,j], _ds.latitude, _ds.pressure_hl, da,
                             dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                             cbar_kwargs={'pad':0.0125, 'aspect':10, 'label':'flux [W m$^{-2}$]'})
        axes[i,j].set_title("Change to downwelling shortwave flux")
        axes[i,j].set_xlabel('')

        i+=1
        da = _ds.flux_up_sw - ds.flux_up_sw
        vmin, vmax = get_vextents(da, symmetric=True)
        irregular_pcolor(axes[i,j], _ds.latitude, _ds.pressure_hl, da,
                             dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                             cbar_kwargs={'pad':0.0125, 'aspect':10, 'label':'flux [W m$^{-2}$]'})
        axes[i,j].set_title("Change to upwelling shortwave flux")
        axes[i,j].set_xlabel('')

        #Cloud radiative effects
        j=0
        i+=1
        da = _ds.cloud_radiative_effect_lw - ds.cloud_radiative_effect_lw
        vmin, vmax = get_vextents(da, symmetric=True)
        irregular_pcolor(axes[i,j], _ds.latitude, _ds.pressure_hl, da,
                             dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                             cbar_kwargs={'pad':0.0125, 'aspect':10, 'label':'CRE$_{\mathrm{LW}}$ [W m$^{-2}$]'})

        axes[i,j].set_xlabel('')
        axes[i,j].set_title("Change to longwave cloud radiative effect")

        j+=1
        da = _ds.cloud_radiative_effect_sw - ds.cloud_radiative_effect_sw
        vmin, vmax= get_vextents(da, symmetric=True)
        irregular_pcolor(axes[i,j], _ds.latitude, _ds.pressure_hl, da,
                             dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                             cbar_kwargs={'pad':0.0125, 'aspect':10, 'label':'CRE$_{\mathrm{SW}}$ [W m$^{-2}$]'})
        axes[i,j].set_xlabel('')
        axes[i,j].set_title("Change to shortwave cloud radiative effect")

        #Heating rates
        cbar_kwargs = {'pad':0.0125, 'aspect':10, 'label':'$\dfrac{dT}{dt}$ [K d$^{-1}$]'}
        j=0
        i+=1

        linear_pressure=True

        da = _ds.heating_rate_lw - ds.heating_rate_lw
        if linear_pressure:
            vmin, vmax= get_vextents(da.where(_ds.pressure_fl > 100e2))
        else:
            vmin, vmax= get_vextents(da)
        irregular_pcolor(axes[i,j], _ds.latitude, _ds.pressure_fl, da,
                         dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                         cbar_kwargs=cbar_kwargs)
        axes[i,j].set_xlabel('')
        axes[i,j].set_title("Change to longwave heating rate (troposphere)")

        j+=1
        da = _ds.heating_rate_sw - ds.heating_rate_sw
        if linear_pressure:
            vmin, vmax= get_vextents(da.where(_ds.pressure_fl > 100e2))
        else:
            vmin, vmax= get_vextents(da)
        irregular_pcolor(axes[i,j], _ds.latitude, _ds.pressure_fl, da,
                         dict(cmap='RdBu_r',vmin=vmin, vmax=vmax),
                         cbar_kwargs=cbar_kwargs)
        axes[i,j].set_xlabel('')
        axes[i,j].set_title("Change to shortwave heating rate (troposphere)")

        j=0
        i+=1

        linear_pressure=False

        da = _ds.heating_rate_lw - ds.heating_rate_lw
        if linear_pressure:
            vmin, vmax= get_vextents(da.where(_ds.pressure_fl > 100e2))
        else:
            vmin, vmax= get_vextents(da)
        irregular_pcolor(axes[i,j], _ds.latitude, _ds.pressure_fl, da,
                         dict(cmap='RdBu_r', vmin=vmin, vmax=vmax),
                         cbar_kwargs=cbar_kwargs)
        axes[i,j].set_xlabel('')
        axes[i,j].set_title("Change to longwave heating rate (stratosphere)")

        j+=1
        da = _ds.heating_rate_sw - ds.heating_rate_sw
        if linear_pressure:
            vmin, vmax= get_vextents(da.where(_ds.pressure_fl > 100e2))
        else:
            vmin, vmax= get_vextents(da)
        irregular_pcolor(axes[i,j], _ds.latitude, _ds.pressure_fl, da,
                         dict(cmap='RdBu_r',vmin=vmin, vmax=vmax),
                         cbar_kwargs=cbar_kwargs)
        axes[i,j].set_xlabel('')
        axes[i,j].set_title("Change to shortwave heating rate (stratosphere)")

        for ax in axes[:-1,:].flatten():
            add_temperature_contours(ax, _ds)
            ax.set_yscale('linear')
            ax.set_yticks([1000e2,800e2,600e2,400e2,200e2,1])
            ax.set_ylim(1050e2,1)

        for ax in axes[-1,:].flatten():
            add_temperature_contours(ax, _ds)
            ax.set_yscale('log')
            ax.set_yticks([1e5,1e4,1e3,1e2,1e1,1e0])
            ax.set_ylim(1.1e5,1)

            ax.set_xlim(-90,90)
            ax.set_xticks(np.arange(-90,91,15))
            format_latitude(ax)
            ax.set_xlabel('Latitude')

        for ax in axes[:,0]:
            format_pressure(ax)

        x = (get_figure_center(axes[0,0]) + get_figure_center(axes[0,1]))/2
        y = get_figure_top(fig, axes[0,0], include_hspace=True)

        name_string = os.path.splitext(os.path.basename(IFS_srcfile))[0]
        fig.suptitle(f"{name_string}\n{_ds.attrs['experiment']} minus {ds.attrs['experiment']}\nFluxes, cloud radiative effects and heating rates",
                     x=x, y=y-0.025, va='bottom', fontsize='xx-large')

        add_subfigure_labels(axes, flatten_order='C') # This will go across columns, then down rows

        if dstfile:
            fig.savefig(dstfile, dpi=90, bbox_inches='tight')
        else:
            return fig, axes


def plot_output_scalar(IFS_srcfile, ecRAD_srcfiles, ecRAD_styles, dstfile=None, latitudes_to_highlight=None, line_ds='default', title=None):

    with sns.axes_style("ticks", {"xtick.major.size": 6, "ytick.major.size": 6}):

        nrows=4
        ncols=2
        fig, axes = plt.subplots(figsize=(11.0*ncols,3.*nrows), nrows=nrows, ncols=ncols, sharex=True, gridspec_kw={'wspace':0.05, 'hspace':0.25})

        main_legend_labels = []
        main_legend_handles = []
        for i, (_style, _srcfile) in enumerate(zip(ecRAD_styles, ecRAD_srcfiles)):
            _ds = load_ecRAD(_srcfile, IFS_srcfile)

            if i == 0:

                # LW upwelling at surface
                _line = (_ds.flux_up_lw.isel(half_level=-1)).plot(ax=axes[0,0], color='0.85', zorder=-1, lw=3, **{'label':'Upwelling longwave\nflux at surface'})[0]
                minor_legend_ax0 = axes[0,0].legend([_line], ['Longwave upwelling\nflux at surface'], loc='upper left', frameon=False, fontsize='x-small')
                for text in minor_legend_ax0.get_texts():
                    text.set_color("0.75")

                #SW downwelling at TOA
                _line = (_ds.flux_dn_direct_sw_clear.isel(half_level=0)).plot(ax=axes[1,1], color='0.85', zorder=-1, lw=3, **{'label':'Shortwave downwelling\nflux at TOA'})[0]
                minor_legend_ax1 = axes[1,1].legend([_line], ['Shortwave downwelling\nflux at TOA'], loc='upper left', frameon=False, fontsize='x-small')
                for text in minor_legend_ax1.get_texts():
                    text.set_color("0.75")

            main_legend_labels.append(_ds.attrs['experiment'])

            # LW up at TOA
            main_legend_handles.append(_ds.flux_up_lw.isel(half_level=0).plot(ax=axes[0,0], x='latitude', drawstyle=line_ds, **{**_style, **{'label':_ds.attrs['experiment']}}))

            # LW down at surface
            _ds.flux_dn_lw.isel(half_level=-1).plot(ax=axes[1,0], x='latitude', drawstyle=line_ds, **{**_style, **{'label':_ds.attrs['experiment']}})

            # LW CRE at TOA
            _ds.cloud_radiative_effect_lw.isel(half_level=0).plot(ax=axes[2,0], x='latitude', drawstyle=line_ds, **{**_style, **{'label':_ds.attrs['experiment']}})

            # Cloud cover
            _ds.cloud_cover_sw.plot(ax=axes[3,0], x='latitude', drawstyle=line_ds, **{**_style, **{'label':_ds.attrs['experiment']}})

            #SW up at TOA
            _ds.flux_up_sw.isel(half_level=0).plot(ax=axes[0,1], x='latitude', drawstyle=line_ds, **{**_style, **{'label':_ds.attrs['experiment']}})

            #SW down at surface
            _ds.flux_dn_sw.isel(half_level=-1).plot(ax=axes[1,1], x='latitude', drawstyle=line_ds, **{**_style, **{'label':_ds.attrs['experiment']}})

            # SW CRE at TOA
            _ds.cloud_radiative_effect_sw.isel(half_level=0).plot(ax=axes[2,1], x='latitude', drawstyle=line_ds, **{**_style, **{'label':_ds.attrs['experiment']}})

            # SW direct dn at surface
            _ds.flux_dn_direct_sw.isel(half_level=-1).plot(ax=axes[3,1], x='latitude', drawstyle=line_ds, **{**_style, **{'label':_ds.attrs['experiment']}})

        for ax in axes[:,1]:
            ax.yaxis.set_label_position("right")
            ax.yaxis.tick_right()

        axes[0,0].set_ylabel('flux [W m$^{-2}$]')
        axes[0,0].set_title('Longwave upwelling flux at TOA', color=sns.color_palette()[3])
        axes[0,0].set_xlabel('')
        axes[0,0].add_artist(minor_legend_ax0)
        axes[0,0].set_ylim(0, None)

        axes[1,0].set_ylabel('flux [W m$^{-2}$]')
        axes[1,0].set_title('Longwave downwelling flux at surface', color=sns.color_palette()[3])
        axes[1,0].set_xlabel('')
        axes[1,0].set_ylim(0, None)

        axes[2,0].set_ylabel('CRE [W m$^{-2}$]')
        axes[2,0].set_title("Longwave cloud radiative effect at TOA", color=sns.color_palette()[3])
        axes[2,0].set_xlabel('')

        axes[0,1].set_ylabel('flux [W m$^{-2}$]')
        axes[0,1].set_title('Shortwave upwelling flux at TOA', color=sns.color_palette()[0])
        axes[0,1].set_xlabel('')

        axes[1,1].set_ylabel('flux [W m$^{-2}$]')
        axes[1,1].set_title('Shortwave downwelling flux at surface', color=sns.color_palette()[0])
        axes[1,1].set_xlabel('')
        axes[1,1].add_artist(minor_legend_ax1)

        axes[2,1].set_ylabel('CRE [W m$^{-2}$]')
        axes[2,1].set_title("Shortwave cloud radiative effect at TOA", color=sns.color_palette()[0])
        axes[2,1].set_xlabel('')

        axes[3,0].set_ylabel('$f_c$ [-]')
        axes[3,0].set_title("Cloud cover", color='0.33')
        axes[3,0].set_ylim(-0.1, 1.1)
        axes[3,0].set_yticks([0, 0.5, 1])
        format_latitude(axes[3,0])
        axes[3,0].set_xlabel('Latitude')

        axes[3,1].set_ylabel('flux [W m$^{-2}$]')
        axes[3,1].set_title("Shortwave direct downwelling at surface", color=sns.color_palette()[0])
        axes[3,1].set_xlabel('')
        format_latitude(axes[3,1])
        axes[3,1].set_xlabel('Latitude')

        add_subfigure_labels(axes, yloc=1.04)

        if len(ecRAD_srcfiles) > 3:
            legend = axes[0,1].legend(frameon=False, loc='upper right', bbox_to_anchor=(1,1.75), fontsize='xx-small', ncol=2)
        else:
            legend = axes[0,1].legend(frameon=False, loc='upper right', bbox_to_anchor=(1,1.75), fontsize='xx-small', ncol=1)

        fig.suptitle("Fluxes and cloud radiative effects\nat top-of-atmosphere and the surface", y=0.985, fontsize='large')

        axes[-1,0].set_xlim(-90,90)
        axes[-1,0].set_xticks(np.arange(-90,91,30)[:-1])
        format_latitude(axes[-1,0])
        axes[-1,0].set_xlabel('Latitude')


        if dstfile:
            fig.savefig(dstfile, dpi=90, bbox_inches='tight')
        else:
            return fig, axes


def compare_output_scalar(IFS_srcfile, ecRAD_srcfiles, reference_ecRAD_srcfile, ecRAD_styles, reference_label="", latitudes_to_highlight=None, line_ds='default', title=None, dstfile=None):

    with sns.axes_style("ticks", {"xtick.major.size": 8, "ytick.major.size": 8}):

        nrows=3
        ncols=2

        fig, axes = plt.subplots(figsize=(11.0*ncols,3.67*nrows), nrows=nrows, ncols=ncols, sharex=True, gridspec_kw={'hspace':0.4, 'wspace':0.05})

        for ax in axes[:,1]:
            ax.yaxis.set_label_position("right")
            ax.yaxis.tick_right()

        ds = load_ecRAD(reference_ecRAD_srcfile, IFS_srcfile)

        for _style, _srcfile in zip(ecRAD_styles, ecRAD_srcfiles):
            _ds = load_ecRAD(_srcfile, IFS_srcfile)

            #Longwave net flux at TOA
            (_ds.flux_net_lw - ds.flux_net_lw).isel(half_level=0).plot(ax=axes[0,0], x='latitude', drawstyle=line_ds,
                                                          **{**_style, **{'label':f"{_ds.attrs['experiment']} $-$ {ds.attrs['experiment']}"}})

            #Longwave net flux at surface
            (_ds.flux_net_lw - ds.flux_net_lw).isel(half_level=-1).plot(ax=axes[1,0], x='latitude', drawstyle=line_ds,
                                                          **{**_style, **{'label':f"{_ds.attrs['experiment']} $-$ {ds.attrs['experiment']}"}})

            #Change to cloud cover
            (_ds.cloud_cover_sw - ds.cloud_cover_sw).plot(ax=axes[2,0], x='latitude', drawstyle=line_ds,
                                                          **{**_style, **{'label':f"{_ds.attrs['experiment']} $-$ {ds.attrs['experiment']}"}})

            #Shortwave net flux at TOA
            (_ds.flux_net_sw - ds.flux_net_sw).isel(half_level=0).plot(ax=axes[0,1], x='latitude', drawstyle=line_ds,
                                                          **{**_style, **{'label':f"{_ds.attrs['experiment']} $-$ {ds.attrs['experiment']}"}})

            #Shortwave net flux at surface
            (_ds.flux_net_sw - ds.flux_net_sw).isel(half_level=-1).plot(ax=axes[1,1], x='latitude', drawstyle=line_ds,
                                                          **{**_style, **{'label':f"{_ds.attrs['experiment']} $-$ {ds.attrs['experiment']}"}})

            #Shortwave direct downward at surface
            (_ds.flux_dn_direct_sw - ds.flux_dn_direct_sw).isel(half_level=-1).plot(ax=axes[2,1], x='latitude', drawstyle=line_ds,
                                                          **{**_style, **{'label':f"{_ds.attrs['experiment']} $-$ {ds.attrs['experiment']}"}})

        axes[0,0].set_ylabel('$\Delta$ flux [W m$^{-2}$]')
        axes[0,0].set_title('Change in net\nlongwave flux at TOA', color=sns.color_palette()[3])
        axes[0,0].set_xlabel('')

        axes[1,0].set_ylabel('$\Delta$ flux [W m$^{-2}$]')
        axes[1,0].set_title('Change in net\nlongwave flux at surface', color=sns.color_palette()[3])
        axes[1,0].set_xlabel('')

        axes[2,0].set_ylabel('$\Delta$ cloud cover [-]')
        axes[2,0].set_title("Change in cloud cover", color='0.33')
        #axes[2,0].set_ylim(0,None)

        axes[0,1].set_ylabel('$\Delta$ flux [W m$^{-2}$]')
        axes[0,1].set_title('Change in net\nshortwave flux at TOA', color=sns.color_palette()[0])
        axes[0,1].set_xlabel('')
        if len(ecRAD_srcfiles) > 2:
            legend = axes[0,1].legend(frameon=False, loc='upper right', bbox_to_anchor=(1.2,1.67), fontsize='xx-small', ncol=2)
        else:
            legend = axes[0,1].legend(frameon=False, loc='upper right', bbox_to_anchor=(1.2,1.67), fontsize='xx-small', ncol=1)

        axes[1,1].set_ylabel('$\Delta$ flux [W m$^{-2}$]')
        axes[1,1].set_title('Change in net\nshortwave flux at surface', color=sns.color_palette()[0])
        axes[1,1].set_xlabel('')

        axes[2,1].set_ylabel('$\Delta$ flux [W m$^{-2}$]')
        axes[2,1].set_title("Change in direct downwelling\nshortwave flux at surface", color=sns.color_palette()[0])

        axes[2,0].set_xlim(-90,90)
        axes[2,0].set_xticks(np.arange(-90,90,30))
        format_latitude(axes[2,0])
        axes[2,0].set_xlabel('Latitude')
        axes[2,1].set_xlabel('Latitude')

        add_subfigure_labels(axes, yloc=1.04)

        fig.suptitle("Fluxes and cloud radiative effects\nat top-of-atmosphere and the surface", y=1.025, fontsize='large')

        if dstfile:
            fig.savefig(dstfile, dpi=90, bbox_inches='tight')
        else:
            return fig, axes



def plot_on_hybrid_pressure_axis(_axes, x, y, linedict, overwriting=False):
    #Log part of the plot
    _axes[0].plot(x, y, **linedict)
    _axes[1].plot(x, y, **linedict)

    if not overwriting:
        _axes[0].set_yscale('log')
        format_pressure(_axes[0], label='')
        _axes[0].set_yticks([1000,100,10,1])
        _axes[0].set_ylim(10000,1)

        #Linear part of the plot
        format_pressure(_axes[1], label='')
        _axes[1].set_yticks(np.linspace(9e4,1e4,5, dtype='float'))
        _axes[1].set_ylim(101000,10000)

        # Hide the right and top spines
        _axes[0].spines['bottom'].set_visible(False)
        _axes[1].spines['top'].set_visible(False)
        _axes[0].spines['bottom'].set_visible(False)
        _axes[1].spines['top'].set_visible(False)

        _axes[0].axhline(10000, lw=3.5, color='0.67', ls='-', zorder=-11)
        _axes[0].text(0.99, 0.03, 'log', color='0.67', fontsize='small', va='bottom', ha='right', transform=_axes[0].transAxes, zorder=-10)
        _axes[1].text(0.99, 0.99, 'linear', color='0.67', fontsize='small', va='top', ha='right', transform=_axes[1].transAxes, zorder=-10)


def label_hybrid_pressure_axes(_axes):
    l = _axes[0].set_ylabel('Pressure [hPa]')
    x, y = l.get_position()
    l.set_position((x, y - 1))


def plot_input_profile(latitude, IFS_srcfile, dstfile=None, title=None):

    from ecradplot import io as eio
    from ecradplot import plot as eplt

    with sns.plotting_context('talk'):
        ncols=4
        nrows=5
        fig, axes = plt.subplots(figsize=(4.5*ncols,11.5), ncols=ncols, nrows=nrows, gridspec_kw={'hspace':0, 'height_ratios':[1,2,1,1,2]})

        _ds = eio.load_inputs(IFS_srcfile).sel(latitude=latitude, method='nearest')

        #Temperature
        i=0
        plot_on_hybrid_pressure_axis(axes[:2,i], _ds.temperature_hl, _ds.pressure_hl, {'lw':5, 'color':sns.color_palette()[3]})

        #Specific humidity
        i+=1
        plot_on_hybrid_pressure_axis(axes[:2,i], _ds.q, _ds.pressure_fl, {'lw':5, 'color':sns.color_palette()[0]})

        #Cloud fraction
        i+=1
        plot_on_hybrid_pressure_axis(axes[:2,i], _ds.cloud_fraction, _ds.pressure_fl, {'lw':5, 'color':'0.5'})

        #Water content
        i+=1
        plot_on_hybrid_pressure_axis(axes[:2,i], 1e6*_ds.q_liquid.where(_ds.q_liquid > 1e-10).fillna(0), _ds.pressure_fl,
                                     {'lw':5, 'color':sns.color_palette()[3], 'label':'liquid'})
        plot_on_hybrid_pressure_axis(axes[:2,i], 1e6*_ds.q_ice.where(_ds.q_ice > 1e-10).fillna(0), _ds.pressure_fl,
                                     {'lw':5, 'color':sns.color_palette()[0], 'label':'ice'}, overwriting=True)

        #Ozone
        i=0
        plot_on_hybrid_pressure_axis(axes[-2:,i], _ds.o3_mmr, _ds.pressure_fl, {'label':'O$_3$', 'lw':5, 'color':sns.color_palette()[1]})

        #O2 + C02 + CH4 + N20 + CFC
        i+=1
        plot_on_hybrid_pressure_axis(axes[-2:,i], _ds.ch4_vmr, _ds.pressure_fl, {'label':'CH$_4$', 'lw':5, 'color':sns.color_palette()[0]})
        plot_on_hybrid_pressure_axis(axes[-2:,i], _ds.n2o_vmr, _ds.pressure_fl, {'label':'N$_2$O', 'lw':5, 'color':sns.color_palette()[3]}, overwriting=True)

        i+=1
        plot_on_hybrid_pressure_axis(axes[-2:,i], 1e6*_ds.co2_vmr, _ds.pressure_fl, {'label':'CO$_2$', 'lw':5, 'color':sns.color_palette()[2]})

        #Aerosols
        i+=1
        plot_on_hybrid_pressure_axis(axes[-2:,i], _ds.sea_salt, _ds.pressure_fl, {'label':'sea salt', 'lw':5, 'color':sns.color_palette()[0]})
        plot_on_hybrid_pressure_axis(axes[-2:,i], _ds.dust, _ds.pressure_fl, {'label':'dust', 'lw':5, 'color':sns.color_palette()[1]}, overwriting=True)
        plot_on_hybrid_pressure_axis(axes[-2:,i], _ds.organics, _ds.pressure_fl, {'label':'org.', 'lw':5, 'color':sns.color_palette()[2]}, overwriting=True)
        plot_on_hybrid_pressure_axis(axes[-2:,i], _ds.black_carbon, _ds.pressure_fl, {'label':'carbon', 'lw':5, 'color':'0.5'}, overwriting=True)
        plot_on_hybrid_pressure_axis(axes[-2:,i], _ds.sulphate, _ds.pressure_fl, {'label':'sulph.', 'lw':5, 'color':sns.color_palette()[3]}, overwriting=True)

        axes[0,0].set_title('Temperature')
        axes[1,0].set_xlabel("$T$ [K]")
        for ax in axes[:2,0]:
            ax.set_xlim(170,320)

        axes[0,1].set_title('Specific\nhumidity')
        axes[1,1].set_xlabel("$q$ [kg kg$^{-1}$]")
        for ax in axes[:2,1]:
            ax.set_xlim(1e-6,1e-1)
            ax.set_xscale('log')

        axes[0,2].set_title('Cloud fraction')
        axes[1,2].set_xlabel('$f_c$ [-]')
        for ax in axes[:2,2]:
            ax.set_xlim(0,1)

        axes[0,3].set_title('Cloud\nwater content')
        axes[1,3].set_xlabel('$q$ [kg kg$^{-1}$]')

        axes[3,0].set_title('Ozone')
        axes[4,0].set_xlabel("$q_i$ [kg kg$^{-1}$]")
        for ax in axes[3:,0]:
            ax.set_xscale('log')
            ax.set_xticks([1e-7,1e-6,1e-5])

        axes[3,1].set_title('Other gases')
        axes[4,1].set_xlabel('$q_i$ [ppm]')
        for ax in axes[3:,1]:
            ax.set_xscale('log')
            ax.set_xticks([1e-7,1e-6,1e-5])

        axes[3,2].set_title('Other gases')
        axes[4,2].set_xlabel('$q_i$ [ppm]')
        for ax in axes[3:2]:
            ax.set_xlim(395,415)

        axes[3,3].set_title('Aerosols')
        for ax in axes[3:,3]:
            ax.set_xscale('log')
            ax.set_xticks([1e-12,1e-9,1e-6])
        axes[4,3].set_xlabel('mixing ratio\n[kg kg$^{-1}$]')

        axes[0,3].legend(frameon=False, fontsize='small')
        axes[3,1].legend(frameon=False, loc='upper right', fontsize='small')
        axes[3,2].legend(frameon=False, loc='upper right', fontsize='small')
        axes[3,3].legend(frameon=False, loc='upper left', fontsize='x-small', handlelength=1.01, bbox_to_anchor=(1,1))

        for ax in axes[2,:]:
            ax.set_axis_off()
            ax.get_xaxis().set_visible(False)
            ax.get_xaxis().set_visible(False)

        name_string = os.path.splitext(os.path.basename(IFS_srcfile))[0]

        label_hybrid_pressure_axes(axes[:2,0])
        label_hybrid_pressure_axes(axes[-2:,0])

        for ax in axes[:, 1:].flatten():
            ax.set_ylabel("")
            ax.set_yticklabels([])

        for ax in axes[0, :].flatten():
            ax.set_xlabel("")
            ax.set_xticklabels([])

        for ax in axes[3, :].flatten():
            ax.set_xlabel("")
            ax.set_xticklabels([])

        add_subfigure_labels(axes[0,:], xloc=0.015, yloc=0.85, zorder=10, flatten_order='C')
        add_subfigure_labels(axes[3,:], xloc=0.015, yloc=0.85, zorder=10, flatten_order='C', label_list=['e','f','g','h'])

        fig.suptitle(f"{name_string}\nIFS cloud, aerosol and radiation fields\nProfile at {fancy_format_latitude(latitude)}", y=0.95, va='bottom', fontsize='x-large')

        if dstfile:
            fig.savefig(dstfile, dpi=90, bbox_inches='tight')
        else:
            return fig, axes



def plot_output_profile(latitude, IFS_srcfile, ecRAD_srcfiles, linedicts, dstfile=None,
                       clearsky_linedict={ 'ls':'--'}):

    with sns.plotting_context('talk'):

        ncols=4
        nrows=5
        fig, axes = plt.subplots(figsize=(4.5*ncols,12), nrows=nrows, ncols=ncols, gridspec_kw={'hspace':0, 'height_ratios':[1,2,1.1,1,2]})

        for j, ecRAD_srcfile in enumerate(ecRAD_srcfiles):
            _ds = load_ecRAD(ecRAD_srcfile, IFS_srcfile).sel(latitude=latitude, method='nearest')

            i = 0
            if j == 0:
                plot_on_hybrid_pressure_axis(axes[:2,i], _ds.flux_dn_lw, _ds.pressure_hl, {**linedicts[j], **{'label':_ds.attrs['experiment']}})
                plot_on_hybrid_pressure_axis(axes[:2,i], _ds.flux_dn_lw_clear, _ds.pressure_hl, {**linedicts[j], **clearsky_linedict, **{'label':f"{_ds.attrs['experiment']}\n(clearsky)"}}, overwriting=True)
                plot_on_hybrid_pressure_axis(axes[:2,i], _ds.flux_up_lw, _ds.pressure_hl, {**linedicts[j], **{'label':'__nolabel__', 'alpha':0.0}}, overwriting=True)
            else:
                plot_on_hybrid_pressure_axis(axes[:2,i], _ds.flux_dn_lw, _ds.pressure_hl, {**linedicts[j], **{'label':_ds.attrs['experiment']}}, overwriting=True)

            i+=1
            if j == 0:
                plot_on_hybrid_pressure_axis(axes[:2,i], _ds.flux_up_lw, _ds.pressure_hl, {**linedicts[j], **{'label':_ds.attrs['experiment']}})
                plot_on_hybrid_pressure_axis(axes[:2,i], _ds.flux_up_lw_clear, _ds.pressure_hl, {**linedicts[j], **clearsky_linedict, **{'label':f"{_ds.attrs['experiment']}\n(clearsky)"}}, overwriting=True)
                plot_on_hybrid_pressure_axis(axes[:2,i], _ds.flux_dn_lw, _ds.pressure_hl, {**linedicts[j], **{'label':'__nolabel__', 'alpha':0.0}}, overwriting=True)
            else:
                plot_on_hybrid_pressure_axis(axes[:2,i], _ds.flux_up_lw, _ds.pressure_hl, {**linedicts[j], **{'label':_ds.attrs['experiment']}}, overwriting=True)

            i+=1
            if j == 0:
                plot_on_hybrid_pressure_axis(axes[:2,i], _ds.cloud_radiative_effect_lw, _ds.pressure_hl, {**linedicts[j], **{'label':_ds.attrs['experiment']}})
            else:
                plot_on_hybrid_pressure_axis(axes[:2,i], _ds.cloud_radiative_effect_lw, _ds.pressure_hl, {**linedicts[j], **{'label':_ds.attrs['experiment']}}, overwriting=True)

            i+=1
            if j == 0:
                plot_on_hybrid_pressure_axis(axes[:2,i], _ds.heating_rate_lw, _ds.pressure_fl, {**linedicts[j], **{'label':_ds.attrs['experiment']}})
                plot_on_hybrid_pressure_axis(axes[:2,i], _ds.heating_rate_lw_clear, _ds.pressure_fl, {**linedicts[j], **clearsky_linedict, **{'label':f"{_ds.attrs['experiment']}\n(clearsky)"}}, overwriting=True)
            else:
                plot_on_hybrid_pressure_axis(axes[:2,i], _ds.heating_rate_lw, _ds.pressure_fl, {**linedicts[j], **{'label':_ds.attrs['experiment']}}, overwriting=True)


            i=0
            if j == 0:
                plot_on_hybrid_pressure_axis(axes[3:,i], _ds.flux_dn_sw, _ds.pressure_hl, {**linedicts[j], **{'label':_ds.attrs['experiment']}})
                plot_on_hybrid_pressure_axis(axes[3:,i], _ds.flux_dn_sw_clear, _ds.pressure_hl, {**linedicts[j], **clearsky_linedict, **{'label':f"{_ds.attrs['experiment']}\n(clearsky)"}}, overwriting=True)
                plot_on_hybrid_pressure_axis(axes[3:,i], _ds.flux_up_sw, _ds.pressure_hl, {**linedicts[j], **{'label':'__nolabel__', 'alpha':0.0}}, overwriting=True)
            else:
                plot_on_hybrid_pressure_axis(axes[3:,i], _ds.flux_dn_sw, _ds.pressure_hl, {**linedicts[j], **{'label':_ds.attrs['experiment']}}, overwriting=True)

            i+=1
            if j == 0:
                plot_on_hybrid_pressure_axis(axes[3:,i], _ds.flux_up_sw, _ds.pressure_hl, {**linedicts[j], **{'label':_ds.attrs['experiment']}})
                plot_on_hybrid_pressure_axis(axes[3:,i], _ds.flux_up_sw_clear, _ds.pressure_hl, {**linedicts[j], **clearsky_linedict, **{'label':f"{_ds.attrs['experiment']}\n(clearsky)"}}, overwriting=True)
                plot_on_hybrid_pressure_axis(axes[3:,i], _ds.flux_dn_sw, _ds.pressure_hl, {**linedicts[j], **{'label':'__nolabel__', 'alpha':0.0}}, overwriting=True)
            else:
                plot_on_hybrid_pressure_axis(axes[3:,i], _ds.flux_up_sw,  _ds.pressure_hl, {**linedicts[j], **{'label':_ds.attrs['experiment']}}, overwriting=True)

            i+=1
            if j == 0:
                plot_on_hybrid_pressure_axis(axes[3:,i], _ds.cloud_radiative_effect_sw, _ds.pressure_hl, {**linedicts[j], **{'label':_ds.attrs['experiment']}})
            else:
                plot_on_hybrid_pressure_axis(axes[3:,i], _ds.cloud_radiative_effect_sw, _ds.pressure_hl, {**linedicts[j], **{'label':_ds.attrs['experiment']}}, overwriting=True)

            i+=1
            if j == 0:
                plot_on_hybrid_pressure_axis(axes[3:,i], _ds.heating_rate_sw, _ds.pressure_fl, {**linedicts[j], **{'label':_ds.attrs['experiment']}})
                plot_on_hybrid_pressure_axis(axes[3:,i], _ds.heating_rate_sw_clear, _ds.pressure_fl, {**linedicts[j], **clearsky_linedict, **{'label':_ds.attrs['experiment']}}, overwriting=True)
            else:
                plot_on_hybrid_pressure_axis(axes[3:,i], _ds.heating_rate_sw,  _ds.pressure_fl, {**linedicts[j], **{'label':_ds.attrs['experiment']}}, overwriting=True)

        heating_rate_min = np.floor(np.min(axes[0,-1].get_xlim() + axes[1,-1].get_xlim() + axes[3,-1].get_xlim() + axes[4,-1].get_xlim()))
        heating_rate_max = np.ceil(np.max(axes[0,-1].get_xlim() + axes[1,-1].get_xlim() + axes[3,-1].get_xlim() + axes[4,-1].get_xlim()))
        heating_rate_abs_lim = np.max([np.abs(heating_rate_min), heating_rate_max])

        for ax in axes[:,:2].flatten():
            ax.set_xlim(0,None)

        axes[0,-1].set_xlim(-1*heating_rate_abs_lim, None)
        axes[1,-1].set_xlim(-1*heating_rate_abs_lim, None)
        axes[3,-1].set_xlim(None, heating_rate_abs_lim)
        axes[4,-1].set_xlim(None, heating_rate_abs_lim)

        axes[0,0].set_title('Downwelling\nlongwave flux', color=sns.color_palette()[3])
        axes[0,1].set_title('Upwelling\nlongwave flux', color=sns.color_palette()[3])
        axes[0,2].set_title('Longwave CRE', color=sns.color_palette()[3])
        axes[0,3].set_title('Longwave\nheating rate', color=sns.color_palette()[3])

        axes[3,0].set_title('Downwelling\nshortwave flux', color=sns.color_palette()[0])
        axes[3,1].set_title('Upwelling\nshortwave flux', color=sns.color_palette()[0])
        axes[3,2].set_title('Shortwave CRE', color=sns.color_palette()[0])
        axes[3,3].set_title('Shortwave\nheating rate', color=sns.color_palette()[0])

        for ax in axes[1,:3]:
            ax.set_xlabel('[W m$^{-2}$]')
        for ax in axes[1,3:]:
            ax.set_xlabel('[K d$^{-1}$]')

        for ax in axes[4,:3]:
            ax.set_xlabel('[W m$^{-2}$]')
        for ax in axes[4,3:]:
            ax.set_xlabel('[K d$^{-1}$]')

        for ax in axes[2,:]:
            ax.set_axis_off()
            ax.get_xaxis().set_visible(False)
            ax.get_xaxis().set_visible(False)

        axes[0,-1].legend(loc='upper left', frameon=False, bbox_to_anchor=(1,1))

        label_hybrid_pressure_axes(axes[:2,0])
        label_hybrid_pressure_axes(axes[-2:,0])

        for ax in axes[:, 1:].flatten():
            ax.set_ylabel("")
            ax.set_yticklabels([])

        for ax in axes[0, :].flatten():
            ax.set_xlabel("")
            ax.set_xticklabels([])

        for ax in axes[3, :].flatten():
            ax.set_xlabel("")
            ax.set_xticklabels([])

        name_string = os.path.splitext(os.path.basename(IFS_srcfile))[0]
        fig.suptitle(f'{name_string}\nProfile at {fancy_format_latitude(latitude)}', y=0.95, va='bottom', fontsize='x-large')

        add_subfigure_labels(axes[0,:], xloc=0.025, yloc=0.85, zorder=10)
        add_subfigure_labels(axes[3,:], xloc=0.025, yloc=0.85, zorder=10, label_list=['e','f','g','h'])

        if dstfile:
            fig.savefig(dstfile, dpi=90, bbox_inches='tight')
        else:
            return fig, axes


def compare_output_profile(latitude, IFS_srcfile, ecRAD_reference_srcfile, ecRAD_srcfiles, linedicts, dstfile=None,
                       clearsky_linedict={'ls':'--'}):

    with sns.plotting_context('talk'):

        ncols=4
        nrows=5
        fig, axes = plt.subplots(figsize=(4.5*ncols,12), nrows=nrows, ncols=ncols, gridspec_kw={'hspace':0, 'height_ratios':[1,2,1.2,1,2]})

        ds = load_ecRAD(ecRAD_reference_srcfile, IFS_srcfile).sel(latitude=latitude, method='nearest')

        for j, ecRAD_srcfile in enumerate(ecRAD_srcfiles):
            _ds = load_ecRAD(ecRAD_srcfile, IFS_srcfile).sel(latitude=latitude, method='nearest')

            i = 0
            plot_on_hybrid_pressure_axis(axes[:2,i], _ds.flux_dn_lw - ds.flux_dn_lw, _ds.pressure_hl,
                                         {**linedicts[j], **{'label':f"{_ds.attrs['experiment']} $-$ {ds.attrs['experiment']}"}})
            if j == 0:
                plot_on_hybrid_pressure_axis(axes[:2,i], _ds.flux_dn_lw_clear - ds.flux_dn_lw_clear, _ds.pressure_hl,
                                         {**linedicts[j], **clearsky_linedict, **{'label':f"{_ds.attrs['experiment']} $-$ {ds.attrs['experiment']}\n(clearsky)"}},  overwriting=True)

            i+=1
            plot_on_hybrid_pressure_axis(axes[:2,i], _ds.flux_up_lw - ds.flux_up_lw, _ds.pressure_hl,
                                         {**linedicts[j], **{'label':f"{_ds.attrs['experiment']} $-$ {ds.attrs['experiment']}"}})
            if j == 0:
                plot_on_hybrid_pressure_axis(axes[:2,i], _ds.flux_up_lw_clear - ds.flux_up_lw_clear, _ds.pressure_hl,
                                         {**linedicts[j], **clearsky_linedict, **{'label':f"{_ds.attrs['experiment']} $-$ {ds.attrs['experiment']}\n(clearsky)"}},  overwriting=True)

            i+=1
            plot_on_hybrid_pressure_axis(axes[:2,i], _ds.cloud_radiative_effect_lw - ds.cloud_radiative_effect_lw, _ds.pressure_hl,
                                         {**linedicts[j], **{'label':f"{_ds.attrs['experiment']} $-$ {ds.attrs['experiment']}"}})

            i+=1
            plot_on_hybrid_pressure_axis(axes[:2,i], _ds.heating_rate_lw - ds.heating_rate_lw, _ds.pressure_fl,
                                         {**linedicts[j], **{'label':f"{_ds.attrs['experiment']} $-$ {ds.attrs['experiment']}"}})
            if j == 0:
                plot_on_hybrid_pressure_axis(axes[:2,i], _ds.heating_rate_lw_clear - ds.heating_rate_lw_clear, _ds.pressure_fl,
                                         {**linedicts[j], **clearsky_linedict, **{'label':f"{_ds.attrs['experiment']} $-$ {ds.attrs['experiment']}\n(clearsky)"}}, overwriting=True)


            i=0
            plot_on_hybrid_pressure_axis(axes[3:,i], _ds.flux_dn_sw - ds.flux_dn_sw, _ds.pressure_hl,
                                         {**linedicts[j], **{'label':f"{_ds.attrs['experiment']} $-$ {ds.attrs['experiment']}"}})
            if j == 0:
                plot_on_hybrid_pressure_axis(axes[3:,i], _ds.flux_dn_sw_clear - ds.flux_dn_sw_clear, _ds.pressure_hl,
                                         {**linedicts[j], **clearsky_linedict, **{'label':f"{_ds.attrs['experiment']} $-$ {ds.attrs['experiment']}\n(clearsky)"}}, overwriting=True)

            i+=1
            plot_on_hybrid_pressure_axis(axes[3:,i], _ds.flux_up_sw - ds.flux_up_sw, _ds.pressure_hl,
                                         {**linedicts[j], **{'label':f"{_ds.attrs['experiment']} $-$ {ds.attrs['experiment']}"}})
            if j == 0:
                plot_on_hybrid_pressure_axis(axes[3:,i], _ds.flux_up_sw_clear - ds.flux_up_sw_clear, _ds.pressure_hl,
                                         {**linedicts[j], **clearsky_linedict, **{'label':f"{_ds.attrs['experiment']} $-$ {ds.attrs['experiment']}\n(clearsky)"}},  overwriting=True)

            i+=1
            plot_on_hybrid_pressure_axis(axes[3:,i], _ds.cloud_radiative_effect_sw - ds.cloud_radiative_effect_sw, _ds.pressure_hl,
                                         {**linedicts[j], **{'label':f"{_ds.attrs['experiment']} $-$ {ds.attrs['experiment']}"}})

            i+=1
            plot_on_hybrid_pressure_axis(axes[3:,i], _ds.heating_rate_sw - ds.heating_rate_sw, _ds.pressure_fl,
                                         {**linedicts[j], **{'label':f"{_ds.attrs['experiment']} $-$ {ds.attrs['experiment']}"}})
            if j == 0:
                plot_on_hybrid_pressure_axis(axes[3:,i], _ds.heating_rate_sw_clear - ds.heating_rate_sw_clear, _ds.pressure_fl,
                                         {**linedicts[j], **clearsky_linedict, **{'label':f"{_ds.attrs['experiment']} $-$ {ds.attrs['experiment']}\n(clearsky)"}},  overwriting=True)


        axes[0,0].set_title('Change to downwelling\nlongwave flux', color=sns.color_palette()[3])
        axes[0,1].set_title('Change to upwelling\nlongwave flux', color=sns.color_palette()[3])
        axes[0,2].set_title('Change to \nlongwave CRE', color=sns.color_palette()[3])
        axes[0,3].set_title('Change to longwave\nheating rate', color=sns.color_palette()[3])

        axes[3,0].set_title('Change to downwelling\nshortwave flux', color=sns.color_palette()[0])
        axes[3,1].set_title('Change to upwelling\nshortwave flux', color=sns.color_palette()[0])
        axes[3,2].set_title('Change to\nshortwave CRE', color=sns.color_palette()[0])
        axes[3,3].set_title('Change to shortwave\nheating rate', color=sns.color_palette()[0])

        for ax in axes[1,:3]:
            ax.set_xlabel('[W m$^{-2}$]')
        for ax in axes[1,3:]:
            ax.set_xlabel('[K d$^{-1}$]')

        for ax in axes[4,:3]:
            ax.set_xlabel('[W m$^{-2}$]')
        for ax in axes[4,3:]:
            ax.set_xlabel('[K d$^{-1}$]')

        for ax in axes[2,:]:
            ax.set_axis_off()
            ax.get_xaxis().set_visible(False)
            ax.get_xaxis().set_visible(False)

        axes[0,-1].legend(loc='upper left', frameon=False, bbox_to_anchor=(1,1))

        label_hybrid_pressure_axes(axes[:2,0])
        label_hybrid_pressure_axes(axes[-2:,0])

        for ax in axes[:, 1:].flatten():
            ax.set_ylabel("")
            ax.set_yticklabels([])

        for ax in axes[0, :].flatten():
            ax.set_xlabel("")
            ax.set_xticklabels([])

        for ax in axes[3, :].flatten():
            ax.set_xlabel("")
            ax.set_xticklabels([])

        name_string = os.path.splitext(os.path.basename(IFS_srcfile))[0]
        fig.suptitle(f'{name_string}\nProfile at {fancy_format_latitude(latitude)}', y=0.95, va='bottom', fontsize='x-large')

        add_subfigure_labels(axes[0,:], xloc=0.015, yloc=0.85, zorder=10)
        add_subfigure_labels(axes[3,:], xloc=0.015, yloc=0.85, zorder=10, label_list=['e','f','g','h'])

        if dstfile:
            fig.savefig(dstfile, dpi=90, bbox_inches='tight')
        else:
            return fig, axes



#!/usr/bin/env python3

def warn(*args, **kwargs):
    pass
    
import os, warnings
warnings.warn = warn

from ecradplot import plot as eplt

def main(latitude, input_srcfile, dstdir):
    """
    Plot input files
    """
    
    import os
    if not os.path.isdir(dstdir):
        os.makedirs(dstdir)

    name_string = os.path.splitext(os.path.basename(input_srcfile))[0]
        
    dstfile = os.path.join(dstdir, name_string + f"_profile_{eplt.unfancy_format_latitude(latitude)}.png")

    print(f"Plotting inputs profile to {dstfile}")
    eplt.plot_input_profile(latitude, input_srcfile, dstfile=dstfile);
    
if __name__ == "__main__":
    import argparse
    parser = argparse.ArgumentParser(description="Plot profiles of atmospheric composition and clouds from input file to ecRAD.")
    parser.add_argument("latitude", help="Latitude at which to extract profiles", type=float)
    parser.add_argument("input",  help="ecRAD input file")
    parser.add_argument("--dstdir", help="Destination directory for plots", default="./")
    args = parser.parse_args()
    
    main(args.latitude, args.input, args.dstdir)



import os
import re

from depgen import file_in_dir


class LCProcessor:
    _re_lc = re.compile(r'^#\s*[1-9]\d*\s*"(.*?)"\s*(?:[1-9]\d*)?')

    def __init__(self, stream,
                 include_roots=None):
        self.include_roots = include_roots

        # Callbacks:
        self.lc_callback = None
        self.debug_callback = None

        self._stream = stream

    def readline(self):
        while 1:
            line = self._stream.readline()
            if not line:
                return line

            match = LCProcessor._re_lc.match(line)
            if match:
                filepath = match.group(1)
                if os.path.isfile(filepath):
                    if not self.include_roots or any(
                            [file_in_dir(filepath, d)
                             for d in self.include_roots]):
                        if self.lc_callback:
                            self.lc_callback(filepath)
                        if self.debug_callback:
                            self.debug_callback(
                                line, 'accepted file \'%s\'' % filepath)
                    elif self.debug_callback:
                        self.debug_callback(
                            line, 'ignored (file \'%s\' '
                                  'is not in the source roots)' % filepath)
                elif self.debug_callback:
                    self.debug_callback(line, 'ignored (file not found)')
                continue

            return line

    @property
    def name(self):
        return self._stream.name

    def close(self):
        self._stream.close()



#!/usr/bin/env python3
import os
import warnings
import seaborn as sns
from ecradplot import plot as eplt

def warn(*args, **kwargs):
    pass

warnings.warn = warn


def format_latitude(latitude):
    if latitude<0:
        return f"{abs(latitude):.0f}S"

    return f"{abs(latitude):.0f}N"

def main(latitude, input_srcfile, output_srcfiles, dstdir):
    """
    Plot input files
    """

    if not os.path.isdir(dstdir):
        os.makedirs(dstdir)

    name_string  = os.path.splitext(os.path.basename(input_srcfile))[0]
    outputs_string = "_".join([os.path.splitext(os.path.basename(f))[0] for f in output_srcfiles])

    styles = [{'lw':3, 'color':'k', 'ls':'-', 'zorder':10},
              #{'lw':4, 'color':'0.5', 'ls':'-'},
              #{'lw':4, 'color':'0.75', 'ls':'-'},
              {'lw':5, 'color':sns.color_palette()[0], 'ls':'-'},
              {'lw':5, 'color':sns.color_palette()[2], 'ls':'-'},
              {'lw':3, 'color':sns.color_palette()[3], 'ls':'--'},
              {'lw':3, 'color':sns.color_palette()[5], 'ls':'-.'},
              {'lw':5, 'color':sns.color_palette()[6], 'ls':'-'},
              {'lw':5, 'color':sns.color_palette()[9], 'ls':'-'}]


    dstfile = os.path.join(
        dstdir,
        f"{name_string}_{outputs_string}_profile_{format_latitude(latitude)}.png"
    )
    print(f"Plotting output profiles to {dstfile}")
    eplt.plot_output_profile(latitude, input_srcfile, output_srcfiles, styles, dstfile=dstfile)

if __name__ == "__main__":
    import argparse
    parser = argparse.ArgumentParser(
        description="Plot radiative fluxes and heating rates from ecRAD output file. \
                    If a reference file is given, plot differences with respect to the reference."
    )
    parser.add_argument("latitude",  help="Latitude at which to extract profiles", type=float)
    parser.add_argument("input",     help="ecRAD input file")
    parser.add_argument("outputs",   help="ecRAD output files", nargs='+')
    parser.add_argument("--dstdir",  help="Destination directory for plots", default="./")
    args = parser.parse_args()

    main(args.latitude, args.input, args.outputs, args.dstdir)



#############################################################################
#
# The purpose of this program is to compare the ISCCP simulator test code 
# output to a reference dataset. Specifically, only the test-based 1/+/- cloud 
# representation is compared.
#
# Coded by Dustin Swales 2014 CIRES/NOAA-ESRL-PSD
#
#############################################################################
import os,sys,numpy

wrkDir, filename = os.path.split(os.path.abspath(__file__))
fileNEW  = wrkDir+'/data/output/'+'stdout'
fileREF  = wrkDir+'/data/output/'+'stdout.expected'
print '###################################################################################'
print 'Comparing: '
print '    stdout'
print 'to the reference file: '
print '    stdout.expected'
print '####################################################################################'

#############################################################################
# Determine which parts of the files to compare
#############################################################################

# Number of lines in reference file
fIDref    = open(fileREF,'r')
flinesRef = fIDref.readlines()
nlinesRef = len(flinesRef)

# Number of lines in new file
fIDnew    = open(fileNEW,'r')
flinesNew = fIDnew.readlines()
nlinesNew = len(flinesNew)

# If files are of different length quit.
if (nlinesRef != nlinesNew):
    print "ERROR:  Reference file and comparison file are of different length"
    print "EXITING!!"
    quit()

# Pull out number of points, levels and columns
fIDref   = open(fileREF,'r')
linesREF = fIDref.readlines()
npoints = int(linesREF[20])
nlevs   = int(linesREF[22])
ncols   = int(linesREF[24])
nconfig = 18 # Hardcoded in for # configurations used (3x3x2)
errorThresh = 1e-5

#############################################################################
# Loop over all lines of file and read in tau histogram, grid-mean statistics and 
# subcolumn results.
#############################################################################
hist_ref       = numpy.zeros(shape=(7,7,npoints,nconfig))
cldArea_ref    = numpy.zeros(shape=(npoints,nconfig))
meanPtop_ref   = numpy.zeros(shape=(npoints,nconfig))
meanTauCld_ref = numpy.zeros(shape=(npoints,nconfig))
meanAlbCld_ref = numpy.zeros(shape=(npoints,nconfig))
meanTb_ref     = numpy.zeros(shape=(npoints,nconfig))
meanTbClr_ref  = numpy.zeros(shape=(npoints,nconfig))
hist_new       = numpy.zeros(shape=(7,7,npoints,nconfig))
cldArea_new    = numpy.zeros(shape=(npoints,nconfig))
meanPtop_new   = numpy.zeros(shape=(npoints,nconfig))
meanTauCld_new = numpy.zeros(shape=(npoints,nconfig))
meanAlbCld_new = numpy.zeros(shape=(npoints,nconfig))
meanTb_new     = numpy.zeros(shape=(npoints,nconfig))
meanTbClr_new  = numpy.zeros(shape=(npoints,nconfig))
readLines1     = 'FALSE'
readLines2     = 'FALSE'
nc             = 0 # Configuration counter
np             = 0 # Point counter
errorcount     = 0
for ij in range(0,nlinesRef):

    # Pull out cloud-top height direction type
    if("adjusted top IR only" in flinesRef[ij]):
        top=int(flinesRef[ij+1].split()[0])

    # Determine when and where to stop reading
    if ("START" in flinesRef[ij] or readLines1 == "TRUE" ):
        readLines1 = "TRUE"
    if ("RESUME" in flinesRef[ij]):
        readLines1 = "FALSE"
        readLines2 = "TRUE"
    if ("STOP" in flinesRef[ij]):
        readLines2 = "FALSE"

    # Read in joint-histogram of tau amd gridbox mean statistics.
    if ("taumin" in flinesRef[ij] and readLines1 == "TRUE"):
        for i1 in range(1,8):
            hist_ref[i1-1,:,np,nc]=flinesRef[ij+i1*2+1].split()
            hist_new[i1-1,:,np,nc]=flinesNew[ij+i1*2+1].split()

        hist_test1 = numpy.max(abs(hist_ref[:,:,np,nc]-hist_new[:,:,np,nc]))
        if (hist_test1 != 0.0):
            print '####################################################################################'
            print "          Differences in joint histograms for configuration #",nc+1
            print "REFERENCE (@line",ij,"): "
            print hist_ref[:,:,np,nc]
            print "NEW (@line",ij,"):       "
            print hist_new[:,:,np,nc]
            errorcount=errorcount+1


        if (top != 2):
            cldArea_ref[np,nc]    = float(flinesRef[ij+22].split()[2])
            meanPtop_ref[np,nc]   = float(flinesRef[ij+23].split()[2])
            meanTauCld_ref[np,nc] = float(flinesRef[ij+24].split()[2])
            meanAlbCld_ref[np,nc] = float(flinesRef[ij+25].split()[2])
            meanTb_ref[np,nc]     = float(flinesRef[ij+26].split()[2])
            meanTbClr_ref[np,nc]  = float(flinesRef[ij+27].split()[2])
            cldArea_new[np,nc]    = float(flinesNew[ij+22].split()[2])
            meanPtop_new[np,nc]   = float(flinesNew[ij+23].split()[2])
            meanTauCld_new[np,nc] = float(flinesNew[ij+24].split()[2])
            meanAlbCld_new[np,nc] = float(flinesNew[ij+25].split()[2])
            meanTb_new[np,nc]     = float(flinesNew[ij+26].split()[2])
            meanTbClr_new[np,nc]  = float(flinesNew[ij+27].split()[2])
        if (top == 2):
            cldArea_ref[np,nc]    = float(flinesRef[ij+20].split()[2])
            meanPtop_ref[np,nc]   = float(flinesRef[ij+21].split()[2])
            meanTauCld_ref[np,nc] = float(flinesRef[ij+22].split()[2])
            meanAlbCld_ref[np,nc] = float(flinesRef[ij+23].split()[2])
            meanTb_ref[np,nc]     = -999.00
            meanTbClr_ref[np,nc]  = -999.00
            cldArea_new[np,nc]    = float(flinesNew[ij+20].split()[2])
            meanPtop_new[np,nc]   = float(flinesNew[ij+21].split()[2])
            meanTauCld_new[np,nc] = float(flinesNew[ij+22].split()[2])
            meanAlbCld_new[np,nc] = float(flinesNew[ij+23].split()[2])
            meanTb_new[np,nc]     = -999.00
            meanTbClr_new[np,nc]  = -999.00
 
        # Compare grid-mean statistics
        stat_test1 = numpy.max(abs(cldArea_ref[np,nc]   - cldArea_new[np,nc]))/abs(cldArea_ref[np,nc])
        stat_test2 = numpy.max(abs(meanPtop_ref[np,nc]  - meanPtop_new[np,nc]))/abs(meanPtop_ref[np,nc])
        stat_test3 = numpy.max(abs(meanTauCld_ref[np,nc]- meanTauCld_new[np,nc]))/abs(meanTauCld_ref[np,nc])
        stat_test4 = numpy.max(abs(meanAlbCld_ref[np,nc]- meanAlbCld_new[np,nc]))/abs(meanAlbCld_ref[np,nc])
        stat_test5 = numpy.max(abs(meanTb_ref[np,nc]    - meanTb_new[np,nc]))/abs(meanTb_ref[np,nc])
        stat_test6 = numpy.max(abs(meanTbClr_ref[np,nc] - meanTbClr_new[np,nc]))/abs(meanTbClr_ref[np,nc])
        if (stat_test1 > errorThresh):
            print '####################################################################################'
            print "          Differences in grid-mean statistics (totalcldarea) for configuration #",nc+1
            print "REFERENCE (@line",ij,"): ",cldArea_ref[np,nc]
            print "NEW (@line",ij,"):       ",cldArea_new[np,nc]
            errorcount=errorcount+1
        if (stat_test2 > errorThresh):
            print '####################################################################################'
            print "          Differences in grid-mean statistics (meanptop) for configuration #",nc+1
            print "REFERENCE (@line",ij,"): ",meanPtop_ref[np,nc]
            print "NEW (@line",ij,"):       ",meanPtop_new[np,nc]
            errorcount=errorcount+1
        if (stat_test3 > errorThresh):
            print '####################################################################################'
            print "          Differences in grid-mean statistics (meantaucld) for configuration #",nc+1
            print "REFERENCE (@line",ij,"): ",meanTauCld_ref[np,nc]
            print "NEW (@line",ij,"):       ",meanTauCld_new[np,nc]
            errorcount=errorcount+1
        if (stat_test4 > errorThresh):
            print '####################################################################################'
            print "          Differences in grid-mean statistics (meanalbedocld) for configuration #",nc+1
            print "REFERENCE (@line",ij,"): ",meanAlbCld_ref[np,nc]
            print "NEW (@line",ij,"):       ",meanAlbCld_new[np,nc]
            errorcount=errorcount+1
        if (stat_test5 > errorThresh):
            print '####################################################################################'
            print "          Differences in grid-mean statistics (meantb) for configuration #",nc+1
            print "REFERENCE (@line",ij,"): ",meanTb_ref[np,nc]
            print "NEW (@line",ij,"):       ",meanTb_new[np,nc]
            errorcount=errorcount+1
        if (stat_test6 > errorThresh):
            print '####################################################################################'
            print "          Differences in grid-mean statistics (meantbclr) for configuration #",nc+1
            print "REFERENCE (@line",ij,"): ",meanTbClr_ref[np,nc]
            print "NEW (@line",ij,"):       ",meanTbClr_new[np,nc]
            errorcount=errorcount+1

        np = np+1
        # Reset point counter and iterate configuration counter
        if (np == 24):
            nc = nc+1
            np = 0

    # Compare cloud subcolumn representation 
    if (readLines2 == "TRUE"):
        if (flinesRef[ij] != flinesNew[ij]):
            print '####################################################################################'
            print "          Differences in subcolumn results for configuration #",nc 
            print "REFERENCE (@line",ij,"): ",flinesRef[ij],
            print "NEW (@line",ij,"):       ",flinesNew[ij],
            errorcount = errorcount+1

#############################################################################
# Print summary information
#############################################################################
if (errorcount == 0):
    print 'Using and error-threshold of ',errorThresh
    print "NO DIFFERENCES WERE FOUND BETWEEN THE REFERENCE FILE AND THE NEW FILE"
if (errorcount != 0):
    print '####################################################################################'
    print 'Using and error-threshold of ',errorThresh
    print errorcount," DIFFERENCES WERE FOUND"



 


            



# (c) British Crown Copyright 2020, the Met Office.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#   * Redistributions of source code must retain the above copyright notice,
#     this list of conditions and the following disclaimer.
#   * Redistributions in binary form must reproduce the above copyright notice,
#     this list of conditions and the following disclaimer in the documentation
#     and/or other materials provided with the distribution.
#   * Neither the name of the Met Office nor the names of its contributors may
#     be used to endorse or promote products derived from this software without
#     specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
# THE POSSIBILITY OF SUCH DAMAGE.

import netCDF4,argparse,sys
import numpy as np
import matplotlib.pyplot as plt
import cartopy.crs as ccrs
import os

def collapse_dimensions_for_plotting(longitude, latitude, vname, vx, vd, dims):
    """
    Pre-processing of COSP variable for plotting.

    Arguments:
        longitude: longitude from COSP output file [loc].
        latitude: latitude from COSP output file [loc].
        vname: variable name.
        vx: variable from COSP output file [..., loc]
        vd: dictionary with metadata about variable.
        dims: dictionary with additional dimensions.
    Return:
        x: x axis values.
        y: y axis values.
        z: data array for plotting.
        d: dictionary with plot configuration.
    """
    yflip = False
    xticks, yticks, xticks_labels, yticks_labels, xlabel, ylabel, vmax = (None,)*7
    if vd['plot_type'] == 'map':
        d = None
        x = longitude[0]
        y = latitude[:,0]
        z = vx
        if vname == 'parasolGrid_refl': z = vx[2]
        # Roll longitude if there are values > 180
        m = (x > 180.0)
        if m is not None:
            Nroll = longitude.shape[1] // 2
            x[m] = x[m] - 360.0
            x = np.roll(x, Nroll)
            z = np.roll(z,Nroll,axis=1)
        # Calculate latitude and longitude edge points.
        # Assume they are increasing monotonically.
        # Extend length to N+2 and calculate midpoints.
        x = midpoints_to_edges(x)
        y = midpoints_to_edges(y)
        xticks = np.arange(-180,181,60)
        yticks = np.arange(-90,91,30)
        xlabel = 'Longitude (deg)'
        ylabel = 'Latitude (deg)'
    if vd['plot_type'] == '2Dhist':
        weights = np.cos(latitude * np.pi / 180.0)
        weights = weights / weights.sum()
        z = np.sum(vx * weights, axis=2)
        x = np.arange(z.shape[1]+1)
        y = np.arange(z.shape[0]+1)
    if vd['plot_type'] == 'zonal_cross_section':
        z = np.average(vx, axis=2)
        x = midpoints_to_edges(latitude[:,0])
        y = np.arange(z.shape[0] + 1)
    if vd['plot_type'] in ('2Dhist','zonal_cross_section'):
        if vd['xaxis_type'] == 'tau7':
            xticks_labels = ('0', '0.3', '1.3', '3.6', '9.4', '23', '60', '')
            xticks = x
            xlabel = 'Cloud optical depth'
        if vd['xaxis_type'] == 'cloudsat_DBZE_BINS':
            x = np.arange(-50,26,5)
            xticks = x
            xticks_labels = None
            xlabel = 'Radar reflectivity (dBZ)'
        if vd['xaxis_type'] == 'SR_BINS':
            xticks_labels = ('0', '0.01', '1.2', '3', '5', '7', '10', '15', '20',
                             '25', '30', '40', '50', '60', '80', '')
            xticks = x
            xlabel = 'Lidar scattering ratio'
        if vd['xaxis_type'] == 'latitude':
            xticks_labels = None
            xticks = np.arange(-90,91,30)
            xlabel = 'Latitude (deg)'          
        if vd['yaxis_type'] == 'pres7':
            yticks_labels = ('1000', '800', '680', '560', '440', '310', '180','')
            yticks = y
            ylabel = 'Cloud Top Pressure (hPa)'
        if vd['yaxis_type'] == 'hgt16':
            yticks_labels =  ('', '0', '500', '1000', '1500', '2000', '2500',
                              '3000', '4000', '5000', '7000', '9000', '11000',
                              '13000', '15000', '17000', '')
            yticks = y
            ylabel = 'Cloud Top Height (m)'
        if vd['yaxis_type'] == 'REICE_MODIS':
            yticks_labels =  ('0','5', '10', '20', '30', '40','50', '60', '10000')
            yticks = y
            ylabel = 'Ice particle size (micron)'
        if vd['yaxis_type'] == 'RELIQ_MODIS':
            yticks_labels = ('0','4', '8', '10', '12.5', '15', '20', '30', '10000')
            yticks = y
            ylabel = 'Liquid particle size (micron)'
        if vd['yaxis_type'] == 'levStat':
            if not dims['levStat'].any():
                # For diagnostics on model levels, all elements in levStat
                # are set to zero. Keep vertical coordinate as level index.
                ylabel = 'Model level'
            else:
                y = np.concatenate(([0.0], dims['levStat'][::-1] + dims['levStat'][-1]))
                ylabel = 'Altitude (m)'
            yticks = y[0::4]
            yticks_labels = None
            yflip = True
        if vd['yaxis_type'] == 'lev':
            yticks = y[0::4]
            yticks_labels = None
            ylabel = 'Model level'
            yflip = True
    # Extra processing for specific variables
    vmax = None
    if vname == 'cfadLidarsr355': vmax = 0.03
    if vname == 'cfadLidarsr532': vmax = 0.03
    if vname == 'cfadLidarsr532gr': vmax = 0.03
    if vname == 'cfadDbze94': vmax = 0.05
    if vname == 'iwpmodis': vmax = 2.0
    if vname == 'lwpmodis': vmax = 1.0
    if vname == 'tauisccp': vmax = 100.0
    if vname == 'tautmodis': vmax = 100.0
    if vname == 'tauwmodis': vmax = 100.0
    d = {'xticks':xticks,
         'yticks':yticks,
         'xticks_labels':xticks_labels,
         'yticks_labels':yticks_labels,
         'xlabel':xlabel,
         'ylabel':ylabel,
         'vmax':vmax}
    # Flip y axis?
    if yflip: z = np.flip(z, axis=0)
    return x, y, z, d

def midpoints_to_edges(x):
    """
    Calculate edge points. Midpoints must increase monotonically.

    Arguments:
        x: vector with mid points. Dimension N.
    Return:
        y: numpy vector with edges. Dimension N+1.
    """
    y = np.append(np.append(2 * x[0] - x[1], x), 2 * x[-1] - x[-2])
    return 0.5 * (y[1:] + y[:-1])

def plot_pcolormesh(x, y, v, d, fig_name, title=None, coastlines=False):
    """
    Plot pcolormesh and write the output to a png file.

    Arguments:
        x: x axis values.
        y: y axis values.
        v: data array. Dimensions [Nx,Ny,Np]
        d: dictionary with plot configuration.
        fig_name: output file name.
    Keywords:
        title: plot title.
        coastlines: plot coast lines.
    """
    fig = plt.figure(figsize=(10,5))
    cmap = plt.get_cmap('YlOrRd', 20)
    cmap.set_bad('grey', 1)
    if coastlines:
        ax = plt.axes(projection=ccrs.PlateCarree(central_longitude=0))
        ax.coastlines()
    h = plt.pcolormesh(x, y, v, cmap=cmap, vmax=d['vmax'])
    if d['xticks_labels'] and len(d['xticks_labels']) == len(d['xticks']):
        plt.xticks(d['xticks'],d['xticks_labels'])
    else:
        plt.xticks(d['xticks'])
    if d['yticks_labels'] and len(d['yticks_labels']) == len(d['yticks']):
        plt.yticks(d['yticks'],d['yticks_labels'])
    else:
        plt.yticks(d['yticks'])
    plt.xlabel(d['xlabel'])
    plt.ylabel(d['ylabel'])
    plt.colorbar(h,orientation='vertical')
    if title is not None: plt.title(title)
    plt.savefig(fig_name, dpi=200)
    plt.close()

def read_dimensions(fname):
    """
    Read useful dimensions from COSP output file.

    Arguments:
        fname: path to NetCDF file.
    Return:
        d: dictionary with the following dimensions:
           'cloudsat_DBZE_BINS', 'hgt16', 'REICE_MODIS',
           'RELIQ_MODIS', 'levStat', 'SR_BINS', 'lev'
    """
    dim_names = ['cloudsat_DBZE_BINS', 'hgt16', 'REICE_MODIS', 'RELIQ_MODIS',
                 'levStat', 'SR_BINS', 'lev']
    d = {}
    f_id = netCDF4.Dataset(fname, 'r')
    for dim in dim_names:
        d[dim] = f_id.variables[dim][:]
    f_id.close()
    return d

def read_var_to_masked_array(fname, vname, fill_value, Nlat_lon = None):
    """
    Reads a variable from a NetCDF file, and produces a masked array.

    Arguments:
        fname: path to NetCDF file.
        vname: variable name.
        fill_value: missing data value.
    Keywords:
        Nlat_lon: tuple (Nrows, Ncols). If defined, variable is
                  reshaped to a lat-lon grid.
    Return:
        x: variable data array.
        lon: longitude array.
        lat: latitude array
        units: units attribute.
        long_name: long name attribute.
    """
    f_id = netCDF4.Dataset(fname, 'r')
    x = np.ma.masked_equal(f_id.variables[vname][:], fill_value)
    lon = np.ma.masked_equal(f_id.variables['longitude'][:], fill_value)
    lat = np.ma.masked_equal(f_id.variables['latitude'][:], fill_value)
    units = f_id.variables[vname].getncattr('units')
    long_name = f_id.variables[vname].getncattr('long_name')
    f_id.close()

    if Nlat_lon is not None:
        x = np.reshape(x, x.shape[:-1]+Nlat_lon)
        lon = np.reshape(lon, lon.shape[:-1] + Nlat_lon)
        lat = np.reshape(lat, lat.shape[:-1] + Nlat_lon)
    return x, lon, lat, units, long_name

def produce_cosp_summary_plots(fname, variables, output_dir, Nlat_lon = None):
    """
    Wrapper function that iterates over a list of COSP variables and produces
    a PNG figure for each of them.

    Arguments:
        fname: COSP output filename.
        variables: list of variable names.
        output_dir: output directory.
    Keywords:
        Nlat_lon: tuple with (lat,lon) dimensions model's grid.
    """
    fill_value = -1.0e30
    dimensions = read_dimensions(fname)
    for vname, vd in variables.items():
        new_shape = None
        if vd['reshape']: new_shape = Nlat_lon
        vx, longitude, latitude, units, long_name = read_var_to_masked_array(fname, vname, fill_value, Nlat_lon = new_shape)
        x, y, z, pkw = collapse_dimensions_for_plotting(longitude, latitude, vname, vx, vd, dimensions)
        title = long_name + ' (' + units + ')'
        fig_name = os.path.join(output_dir, ".".join([os.path.basename(fname), vname, 'png']))
        coastlines = False
        if vd['plot_type'] == 'map': coastlines = True
        plot_pcolormesh(x, y, z, pkw, fig_name, title=title, coastlines=coastlines)

def variable2D_metadata(var_list, fname):
    """
    Return dictionary with metadata for each variable.

    Arguments:
        var_list: list of variable names.
        fname: COSP output filename.
    Return:
        d: dictionary of dictionaries with relevant metadata.
    """
    map_dims = (('loc',),('PARASOL_NREFL','loc'))
    hist2D_dims = (('pres7', 'tau7', 'loc'),
                  ('levStat', 'cloudsat_DBZE_BINS', 'loc'),
                  ('hgt16', 'tau7', 'loc'),
                  ('REICE_MODIS', 'tau7', 'loc'),
                  ('RELIQ_MODIS', 'tau7', 'loc'),
                  ('levStat', 'SR_BINS', 'loc'))
    zcs_dims = (('levStat','loc'), ('lev','loc'))
    f_id = netCDF4.Dataset(fname, 'r')
    vmeta = {}
    print("=== Processing variables in output file:\n {}".format(fname))
    for vname in var_list:
        try:
            x = f_id.variables[vname]
            # Standard map
            if x.dimensions in map_dims:
                vmeta[vname] = {'plot_type':'map', 'reshape':True}
            # 2D histograms
            if x.dimensions in hist2D_dims:
                vmeta[vname] = {'plot_type':'2Dhist', 'reshape':False,
                                'xaxis_type': x.dimensions[1],
                                'yaxis_type': x.dimensions[0]}
            # Zonal cross section
            if x.dimensions in zcs_dims:
                vmeta[vname] = {'plot_type':'zonal_cross_section', 'reshape':True,
                                'xaxis_type': 'latitude',
                                'yaxis_type': x.dimensions[0]}
        except:
            print("Skipping {}, not found in output file.".format(vname))
    f_id.close()
    return vmeta



#######################
# Main
#######################
if __name__ == '__main__':

    # Command line arguments.
    parser = argparse.ArgumentParser()
    parser.add_argument("--tst_file", default="./data/outputs/UKMO/cosp2_output.um_global.nc",
                        help="Test output file.")
    parser.add_argument("--out_dir", default="./data/outputs/UKMO",
                        help="Output directory.")
    parser.add_argument("--Nlat", type=int, default=36, help="Number of latitude points.")
    parser.add_argument("--Nlon", type=int, default=48, help="Number of longitude points.")
    args = parser.parse_args()

    # Dictonaries with list of variables.
    v2D_maps_names = ['cllcalipsoice', 'clmcalipsoice', 'clhcalipsoice', 'cltcalipsoice', 'cllcalipsoliq', 'clmcalipsoliq',
                      'clhcalipsoliq', 'cltcalipsoliq', 'cllcalipsoun', 'clmcalipsoun', 'clhcalipsoun', 'cltcalipsoun',
                      'cllcalipso', 'clmcalipso', 'clhcalipso', 'cltcalipso', 'clopaquecalipso', 'clthincalipso',
                      'clzopaquecalipso', 'clopaquetemp', 'clthintemp', 'clzopaquetemp', 'clopaquemeanz', 'clthinmeanz',
                      'clthinemis', 'clopaquemeanzse', 'clthinmeanzse', 'clzopaquecalipsose', 'cllgrLidar532',
                      'clmgrLidar532', 'clhgrLidar532', 'cltgrLidar532', 'cllatlid', 'clmatlid', 'clhatlid',
                      'cltatlid', 'cloudsatpia', 'cltisccp', 'meantbisccp', 'meantbclrisccp', 'pctisccp', 'tauisccp',
                      'albisccp', 'misr_meanztop', 'misr_cldarea', 'cltmodis', 'clwmodis', 'climodis', 'clhmodis',
                      'clmmodis', 'cllmodis', 'tautmodis', 'tauwmodis', 'tauimodis', 'tautlogmodis', 'tauwlogmodis',
                      'tauilogmodis', 'reffclwmodis', 'reffclimodis', 'pctmodis', 'lwpmodis', 'iwpmodis',
                      'cltlidarradar', 'cloudsat_tcc', 'cloudsat_tcc2','parasolGrid_refl']
    v2D_hists_names = ['clisccp', 'clmodis', 'cfadDbze94', 'clMISR',
                       'modis_Optical_Thickness_vs_ReffICE',
                       'modis_Optical_Thickness_vs_ReffLIQ',
                       'cfadLidarsr532', 'cfadLidarsr532gr', 'cfadLidarsr355']
    v2D_zcs_names = ['clcalipsoice','clcalipsoliq','clcalipsoun','clcalipsotmp','clcalipsotmpice','clcalipsotmpliq',
                     'clcalipsotmpun','clcalipso','clcalipsoopaque','clcalipsothin','clcalipsozopaque',
                     'clcalipsoopacity','clgrLidar532','clatlid','clcalipso2',
                     'lidarBetaMol532gr','lidarBetaMol532','lidarBetaMol355']
    v2D_all_names = v2D_maps_names + v2D_hists_names + v2D_zcs_names
    # Plots for these variables are not yet developed
    # atb532_perp(lev, cosp_scol, loc);
    # atb532(lev, cosp_scol, loc);
    # calipso_tau(lev, cosp_scol, loc);
    # atb532gr(lev, cosp_scol, loc);
    # atb355(lev, cosp_scol, loc);
    # dbze94(lev, cosp_scol, loc);
    # parasolPix_refl(PARASOL_NREFL, cosp_scol, loc);
    # boxtauisccp(cosp_scol, loc);
    # boxptopisccp(cosp_scol, loc);

    # Build dictionary with metadata and produce plots.
    vars2D  = variable2D_metadata(v2D_all_names, args.tst_file)
    produce_cosp_summary_plots(args.tst_file, vars2D, args.out_dir,
                               Nlat_lon=(args.Nlat, args.Nlon))

    print("===== Summary plots produced =====")
